from dungeonsheets.spells.spells import Spell


class Fabricate(Spell):
    """You convert raw materials into products of the same material.
    For example, you
    can fabricate a wooden bridge from a clump of trees, a rope from a patch of
    hemp, and clothes from flax or wool.

    Choose raw materials that you can see
    within range. You can fabricate a Large or smaller object (contained within a
    10-foot cube, or eight connected 5-foot cubes), given a sufficient quantity of
    raw material. If you are working with metal, stone, or another mineral
    substance, however, the fabricated object can be no larger than Medium
    (contained within a single 5-foot cube). The quality of objects made by the
    spell is commensurate with the quality of the raw materials.

    Creatures or
    magic items can't be created or transmuted by this spell. You also can't use it
    to create items that ordinarily require a high degree of craftsmanship, such as
    jewelry, weapons, glass, or armor, unless you have proficiency with the type of
    artisan's tools used to craft such objects.
    """

    name = "Fabricate"
    level = 4
    casting_time = "10 minutes"
    casting_range = "120 feet"
    components = ("V", "S")
    materials = ""
    duration = "Instantaneous"
    ritual = False
    magic_school = "Transmutation"
    classes = ("Wizard",)


class FaerieFire(Spell):
    """Each object in a 20-foot cube within range is outlined in blue, green, or violet
    light (your choice).
    Any creature in the area when the spell is cast is also
    outlined in light if it fails a Dexterity saving throw. For the duration,
    objects and affected creatures shed dim light in a 10-foot radius.

    Any attack
    roll against an affected creature or object has advantage if the attacker can
    see it, and the affected creature or object can't benefit from being invisible.
    """

    name = "Faerie Fire"
    level = 1
    casting_time = "1 action"
    casting_range = "60 feet"
    components = ("V",)
    materials = ""
    duration = "Concentration, up to 1 minute"
    ritual = False
    magic_school = "Evocation"
    classes = ("Bard", "Druid")


class FalseLife(Spell):
    """Bolstering yourself with a necromantic facsimile of life, you gain 1d4 + 4
    temporary hit points for the duration.

    **At Higher Levels:** When you cast this
    spell using a spell slot of 2nd level or higher, you gain 5 additional temporary
    hit points for each slot level above 1st.
    """

    name = "False Life"
    level = 1
    casting_time = "1 action"
    casting_range = "Self"
    components = ("V", "S", "M")
    materials = "A small amount of alcohol or distilled spirits"
    duration = "1 hour"
    ritual = False
    magic_school = "Necromancy"
    classes = ("Sorcerer", "Wizard")


class FarStep(Spell):
    """You teleport up to 60 feet to an unoccupied space you can see. On each of your
    turns before the spell ends, you can use a bonus action to teleport in this way
    again.
    """

    name = "Far Step"
    level = 5
    casting_time = "1 bonus action"
    casting_range = "Self"
    components = ("V",)
    materials = ""
    duration = "Concentration, up to 1 minute"
    ritual = False
    magic_school = "Conjuration"
    classes = ("Sorcerer", "Warlock", "Wizard")


class Fear(Spell):
    """You project a phantasmal image of a creature's worst fears. Each creature in a
    30-foot cone must succeed on a Wisdom saving throw or drop whatever it is
    holding and become frightened for the duration.

    While frightened by this
    spell, a creature must take the Dash action and move away from you by the safest
    available route on each of its turns, unless there is nowhere to move. If the
    creature ends its turn in a location where it doesn't have line of sight to you,
    the creature can make a Wisdom saving throw. On a successful save, the spell
    ends for that creature.
    """

    name = "Fear"
    level = 3
    casting_time = "1 action"
    casting_range = "Self (30-foot cone)"
    components = ("V", "S", "M")
    materials = "A white feather or the heart of a hen"
    duration = "Concentration, up to 1 minute"
    ritual = False
    magic_school = "Illusion"
    classes = ("Bard", "Sorcerer", "Warlock", "Wizard")


class FeatherFall(Spell):
    """Reaction: When you or a creature within 60 feet of you falls

    Choose up to five
    falling creatures within range. A falling creature's rate of descent slows to
    60 feet per round until the spell ends. If the creature lands before the spell
    ends, it takes no falling damage and can land on its feet, and the spell ends
    for that creature.
    """

    name = "Feather Fall"
    level = 1
    casting_time = "Special"
    casting_range = "60 feet"
    components = ("V", "M")
    materials = "A small feather or piece of down"
    duration = "1 minute"
    ritual = False
    magic_school = "Transmutation"
    classes = ("Bard", "Sorcerer", "Wizard")


class Feeblemind(Spell):
    """You blast the mind of a creature that you can see within range, attempting to
    shatter its intellect and personality. The target takes 4d6 psychic damage and
    must make an Intelligence saving throw.

    On a failed save, the creature's
    Intelligence and Charisma scores become 1. The creature can't cast spells,
    activate magic items, understand language, or communicate in any intelligible
    way. The creature can, however, identify its friends, follow them, and even
    protect them.

    At the end of every 30 days, the creature can repeat its saving
    throw against this spell. If it succeeds on its saving throw, the spell ends.

    The spell can also be ended by greater restoration, heal or wish.
    """

    name = "Feeblemind"
    level = 8
    casting_time = "1 action"
    casting_range = "150 feet"
    components = ("V", "S", "M")
    materials = "A handful of clay, crystal, glass, or mineral spheres"
    duration = "Instantaneous"
    ritual = False
    magic_school = "Enchantment"
    classes = ("Bard", "Druid", "Warlock", "Wizard")


class FeignDeath(Spell):
    """You touch a willing creature and put it into a cataleptic state that is
    indistinguishable from death.

    For the spell's duration, or until you use an
    action to touch the target and dismiss the spell, the target appears dead to all
    outward inspection and to spells used to determine the target's status. The
    target is blinded and incapacitated, and its speed drops to 0.
    The target has
    resistance to all damage except psychic damage. If the target is diseased or
    poisoned when you cast the spell, or becomes diseased or poisoned while under
    the spell's effect, the disease and poison have no effect until the spell ends.
    """

    name = "Feign Death"
    level = 3
    casting_time = "1 action"
    casting_range = "Touch"
    components = ("V", "S", "M")
    materials = "A pinch of graveyard dirt"
    duration = "1 hour"
    ritual = True
    magic_school = "Necromancy"
    classes = ("Bard", "Cleric", "Druid", "Wizard")


class FindFamiliar(Spell):
    """You gain the service of a familiar, a spirit that takes an animal form you
    choose: bat, cat, crab, frog (toad), hawk. lizard, octopus, owl, poisonous
    snake, fish (quipper), rat, raven, sea horse, spider, or weasel. Appearing in an
    unoccupied space within range, the familiar has the statistics of the chosen
    form, though it is a celestial, fey or fiend (your choice) instead of a beast.


    Your familiar acts independently of you, but it always obeys your commands.
    In combat, it rolls its own initiative and acts on its own turn. A familiar
    can't attack, but it can take other actions as normal.

    When the familiar drops
    to 0 hit points, it disappears, leaving behind no physical form. It reappears
    after you cast this spell again.

    While your familiar is within 100 feet of
    you, you can communicate with it telepathically. Additionally, as an action, you
    can see through your familiar's eyes and hear what it hears until the start of
    your next turn, gaining the benefits of any special senses that the familiar
    has. During this time, you are deaf and blind with regard to your own senses.


    As an action, you can temporarily dismiss your familiar. It disappears into a
    pocket dimension where it awaits you summons. Alternatively, you can dismiss it
    forever. As an action while it is temporarily dismissed, you can cause it to
    reappear in any unoccupied space within 30 feet of you.

    You can't have more
    than one familiar at a time. If you cast this spell while you already have a
    familiar, you instead cause it to adopt a new form. Choose one of the forms from
    the above list. Your familiar transforms into the chosen creature.

    Finally,
    when you cast a spell with a range of touch, your familiar can deliver the spell
    as if it had cast the spell. Your familiar must be within 100 feet of you, and
    it must use its reaction to deliver the spell when you cast it. If the spell
    requires an attack roll, you use your attack modifier for the roll.
    """

    name = "Find Familiar"
    level = 1
    casting_time = "1 hour"
    casting_range = "10 feet"
    components = ("V", "S", "M")
    materials = (
        "10 gp worth of charcoal, incense, and herbs that must be consumed by fire in a"
        " brass brazier"
    )
    duration = "Instantaneous"
    ritual = True
    magic_school = "Conjuration"
    classes = ("Wizard",)


class FindGreaterSteed(Spell):
    """You summon a spirit that assumes the form of a loyal, majestic mount. Appearing
    in an unoccupied space within range, the spirit takes on a form you choose: a
    griffon, a pegasus, a peryton, a dire wolf, a rhinoceros, or a saber-toothed
    tiger. The creature has the statistics provided in the Monster Manual for the
    chosen form, though it is a celestial, a fey, or a fiend (your choice) instead
    of its normal creature type. Additionally, if it has an Intelligence score of 5
    or lower, its Intelligence becomes 6, and it gains the ability to understand one
    language of your choice that you speak. You control the mount in combat. While
    the mount is within 1 mile of you, you can communicate with it te1epathically.
    While mounted on it, you can make any spell you cast that targets only you also
    target the mount. The mount disappears temporarily when it drops to 0 hit points
    or when you dismiss it as an action. Casting this spell again re-summons the
    bonded mount, with all its hit points restored and any conditions removed. You
    can't have more than one mount bonded by this spell or find steed at the same
    time. As an action, you can release a mount from its bond, causing it to
    disappear permanently. Whenever the mount disappears, it leaves behind any
    objects it was wearing or carrying.
    """

    name = "Find Greater Steed"
    level = 4
    casting_time = "10 minutes"
    casting_range = "30 feet"
    components = ("V", "S")
    materials = ""
    duration = "Instantaneous"
    ritual = False
    magic_school = "Conjuration"
    classes = ("Paladin",)


class FindSteed(Spell):
    """You summon a spirit that assumes the form of an unusually intelligent, strong,
    and loyal steed, creating a long-lasting bond with it. Appearing in an
    unoccupied space within range, the steed takes on a form that you choose, such
    as a warhorse, a pony, a camel, an elk, or a mastiff. (Your DM might allow other
    animals to be summoned as steeds.) The steed has the statistics of the chosen
    form, though it is a celestial, fey, or fiend (your choice) instead of its
    normal type. Additionally, if your steed has an Intelligence of 5 or less, its
    Intelligence becomes 6, and it gains the ability to understand one language of
    your choice that you speak.

    Your steed serves you as a mount, both in combat
    and out, and you have an instinctive bond with it that allows you to fight as a
    seamless unit. While mounted on your steed, you can make any spell you cast that
    targets only you also target your steed.

    When the steed drops to 0 hit points,
    it disappears, leaving behind no physical form. You can also dismiss your steed
    at any time as an action, causing it to disappear. In either case, casting this
    spell again summons the same steed, restored to its hit point maximum.

    While
    your steed is within 1 mile of you, you can communicate with it telepathically.
    You can't have more than one steed bonded by this spell at a time. As an action,
    you can release the steed from its bond at any time, causing it to disappear.
    """

    name = "Find Steed"
    level = 2
    casting_time = "10 minutes"
    casting_range = "30 feet"
    components = ("V", "S")
    materials = ""
    duration = "Instantaneous"
    ritual = False
    magic_school = "Conjuration"
    classes = ("Paladin",)


class FindThePath(Spell):
    """This spell allows you to find the shortest, most direct physical route to a
    specific fixed location that you are familiar with on the same plane of
    existence. If you name a destination on another plan of existence, a destination
    that moves (such as a mobile fortress), or a destination that isn't specific
    (such as "a green dragon's lair"), the spell fails.

    For the duration, as long
    as you are on the same plane of existence as the destination, you know how far
    it is and in what direction it lies. While you are traveling there, whenever you
    are presented with a choice of paths along the way, you atomatically determine
    which path is the shortest and most direct route (but not necessarily the safest
    route) to the destination."
    """

    name = "Find The Path"
    level = 6
    casting_time = "1 minute"
    casting_range = "Self"
    components = ("V", "S", "M")
    materials = (
        "A set of divinatory tools – such as bones, ivory sticks, cards, teeth, or"
        " carved runes – worth 100 gp and an object from the location you wish to find"
    )
    duration = "Concentration, up to 1 day"
    ritual = False
    magic_school = "Divination"
    classes = ("Bard", "Cleric", "Druid")


class FindTraps(Spell):
    """You sense the presence of any trap within range that is within line of sight.
    A
    trap, for the purpose of this spell, includes anything that would inflict a
    sudden or unexpected effect you consider harmful or undesirable, which was
    specifically intended as such by its creator. Thus, the spell would sense an
    area affected by the alarm spell, a glyph of warding, or a mechanical pit trap,
    but it would not reveal a natural weakness in the floor, an unstable ceiling, or
    a hidden sinkhole.

    This spell merely reveals that a trap is present. You don't
    learn the location of each trap, but you do learn the general nature of the
    danger posed by a trap you sense.
    """

    name = "Find Traps"
    level = 2
    casting_time = "1 action"
    casting_range = "120 feet"
    components = ("V", "S")
    materials = ""
    duration = "Instantaneous"
    ritual = False
    magic_school = "Divination"
    classes = ("Cleric", "Druid", "Ranger")


class FingerOfDeath(Spell):
    """You send negative energy coursing through a creature that you can see within
    range, causing it searing pain.
    The target must make a Constitution saving
    throw. It takes 7d8 + 30 necrotic damage on a failed save, or half as much
    damage on a successful one.

    A humanoid killed by this spell rises at the start
    of your next turn as a zombie that is permanently under your command, following
    your verbal orders to the best of its ability.
    """

    name = "Finger Of Death"
    level = 7
    casting_time = "1 action"
    casting_range = "60 feet"
    components = ("V", "S")
    materials = ""
    duration = "Instantaneous"
    ritual = False
    magic_school = "Necromancy"
    classes = ("Sorcerer", "Warlock", "Wizard")


class FireBolt(Spell):
    """You hurl a mote of fire at a creature or object within range. Make a ranged
    spell attack against the target. On a hit, the target takes 1d10 fire damage. A
    flammable object hit by this spell ignites if it isn't being worn or carried.


    **At Higher Levels:** This spell's damage increases by 1d10 when you reach 5th level
    (2d10), 11th level (3d10), and 17th level (4d10).
    """

    name = "Fire Bolt"
    level = 0
    casting_time = "1 action"
    casting_range = "120 feet"
    components = ("V", "S")
    materials = ""
    duration = "Instantaneous"
    ritual = False
    magic_school = "Evocation"
    classes = ("Sorcerer", "Wizard")


class FireShield(Spell):
    """Thin and wispy flames wreathe your body for the duration, shedding bright light
    in a 10-foot radius and dim light for an additional 10 feet, You can end the
    spell early by using an action to dismiss it.

    The flames provide you with a
    warm shield or a chill shield, as you choose. The warm shield grants you
    resistance to cold damage, and the chill shield grants you resistance to fire
    damage.

    In addition, whenever a creature within 5 feet of you hits you with a
    melee attack, the shield erupts with flame. The attacker takes 2d8 fire damage
    from a warm shield, or 2d8 cold damage from a cold shield.
    """

    name = "Fire Shield"
    level = 4
    casting_time = "1 action"
    casting_range = "Self"
    components = ("V", "S", "M")
    materials = "A bit of phosphorus or a firefly"
    duration = "10 minutes"
    ritual = False
    magic_school = "Evocation"
    classes = ("Wizard",)


class FireStorm(Spell):
    """A storm made up of sheets of roaring flame appears in a location you choose
    within range.
    The area of the storm consists of up to ten 10-foot cubes, which
    you can arrange as you wish. Each cube must have at least one face adjacent to
    the face of another cube. Each creature in the area must make  Dexterity saving
    throw. It takes 7d10 fire damage on a failed save, or half as much damage on a
    successful one.

    The fire damages objects in the area and ignites flammable
    objects that aren't being worn or carried. If you choose, plant life in the area
    is unaffected by this spell.
    """

    name = "Fire Storm"
    level = 7
    casting_time = "1 action"
    casting_range = "150 feet"
    components = ("V", "S")
    materials = ""
    duration = "Instantaneous"
    ritual = False
    magic_school = "Evocation"
    classes = ("Cleric", "Druid", "Sorcerer")


class Fireball(Spell):
    """A bright streak flashes from your pointing finger to a point you choose within
    range then blossoms with a low roar into an explosion of flame. Each creature in
    a 20-foot radius must make a Dexterity saving throw. A target takes 8d6 fire
    damage on a failed save, or half as much damage on a successful one. The fire
    spreads around corners. It ignites flammable objects in the area that aren't
    being worn or carried.

    **At Higher Levels:** When you cast this spell using a spell slot
    of 4th level or higher, the damage increases by 1d6 for each slot
    level above 3rd.

    """

    name = "Fireball"
    level = 3
    casting_time = "1 action"
    casting_range = "150 feet"
    components = ("V", "S", "M")
    materials = "A tiny ball of bat guano and sulfur"
    duration = "Instantaneous"
    ritual = False
    magic_school = "Evocation"
    classes = ("Sorcerer", "Wizard")


class FlameArrows(Spell):
    """You touch a quiver containing arrows or bolts. When a target is hit by a ranged
    weapon attack using a piece of ammunition drawn from the quiver, the target
    takes an extra 1d6 fire damage. The spell's magic ends on the piece of
    ammunition when it hits or misses, and the spell ends when twelve pieces of
    ammunition have been drawn from the quiver.

    **At Higher Levels:** When you cast
    this spell using a spell slot of 4th level or higher, the number of pieces of
    ammunition you can affect with this spell increases by two for each slot level
    above 3rd.
    """

    name = "Flame Arrows"
    level = 3
    casting_time = "1 action"
    casting_range = "Touch"
    components = ("V", "S")
    materials = ""
    duration = "Concentration, up to 1 hour"
    ritual = False
    magic_school = "Transmutation"
    classes = ("Druid", "Ranger", "Sorcerer", "Wizard")


class FlameBlade(Spell):
    """You evoke a fiery blade in your free hand.
    The blade is similar in size and
    shape to a scimitar, and it lasts for the duration. If you let go of the blade,
    it disappears, but you can evoke the blade again as a bonus action.

    You can use
    your action to make a melee spell attack with the fiery blade. On a hit, the
    target takes 3d6 fire damage.
    The flaming blade sheds bright light in a 10-foot
    radius and dim light for an additional 10 feet.

    **At Higher Levels:** When you
    cast this spell using a spell slot of 4th level or higher, the damage increases
    by 1d6 for every two slot levels above 2nd.
    """

    name = "Flame Blade"
    level = 2
    casting_time = "1 bonus action"
    casting_range = "Self"
    components = ("V", "S", "M")
    materials = "Leaf of sumac"
    duration = "Concentration, up to 10 minutes"
    ritual = False
    magic_school = "Evocation"
    classes = ("Druid",)


class FlameStrike(Spell):
    """A vertical column of divine fire roars down from the heavens in a location you
    specify. Each creature in a 10-foot radius, 40-foot-high cylinder centered on a
    point within range must make a Dexterity saving throw. A creature takes 4d6 fire
    damage and 4d6 radiant damage on a failed save, or half as much damage on a
    successful one.

    **At Higher Levels:** When you cast this spell using a spell slot
    of 6th level or higher, the fire damage or the radiant damage (your choice)
    increases by 1d6 for each slot level above 5th.
    """

    name = "Flame Strike"
    level = 5
    casting_time = "1 action"
    casting_range = "60 feet"
    components = ("V", "S", "M")
    materials = "Pinch of sulfur"
    duration = "Instantaneous"
    ritual = False
    magic_school = "Evocation"
    classes = ("Cleric",)


class FlamingSphere(Spell):
    """A 5-foot-diameter sphere of fire appears in an unoccupied space of your choice
    within range and lasts for the duration.
    Any creature that ends its turn within
    5 feet of the sphere must make a Dexterity saving throw. The creature takes 2d6
    fire damage on a failed save, or half as much damage on a successful one.

    As a
    bonus action, you can move the sphere up to 30 feet. If you ram the sphere into
    a creature, that creature must make the saving throw against the sphere's
    damage, and the sphere stops moving this turn.

    When you move the sphere, you
    can direct it over barriers up to 5 feet tall and jump it across pits up to 10
    feet wide. The sphere ignites flammable objects not being worn or carried, and
    it sheds bright light in a 20-foot radius and dim light for an additional 20
    feet.

    **At Higher Levels:** When you cast this spell using a spell slot of 3rd
    level or higher, the damage increases by 1d6 for each slot level above 2nd.
    """

    name = "Flaming Sphere"
    level = 2
    casting_time = "1 action"
    casting_range = "60 feet"
    components = ("V", "S", "M")
    materials = (
        """A bit of tallow, a pinch of brimstone, and a dusting of powdered iron"""
    )
    duration = "Concentration, up to 1 minute"
    ritual = False
    magic_school = "Conjuration"
    classes = ("Druid", "Wizard")


class FleshToStone(Spell):
    """You attempt to turn one creature that you can see within range into stone.
    If
    the targets body is made of flesh, the creature must make a Constitution saving
    throw. On a failed save, it is restrained as its flesh begins to harden. On a
    successful save, the creature isn't affected.

    A creature restrained by this
    spell must make another Consititution saving throw at the end of each of its
    turns. If it successfully saves against this spell three times, the spell ends.
    If it fails saves three times, it is turned to stone and subjected to the
    petrified condition for the duration. The successes and failures don't need to
    be consecutive; keep track of both until the target collects three of a kind.


    If the creature is physically broken while petrified, it suffers from similar
    deformities if it reverts to its original state. If you maintain your
    concentration on this spell for the entire possible duration, the creature is
    turned to stone until the effect is removed.
    """

    name = "Flesh To Stone"
    level = 6
    casting_time = "1 action"
    casting_range = "60 feet"
    components = ("V", "S", "M")
    materials = "A pinch of lime, water, and earth"
    duration = "Concentration, up to 1 minute"
    ritual = False
    magic_school = "Transmutation"
    classes = ("Warlock", "Wizard")


class Fly(Spell):
    """You touch a willing creature. The target gains a flying speed of 60 feet for the
    duration. When the spell ends, the target falls if it is still aloft, unless it
    can stop the fall.

    **At Higher Levels:** When you cast this spell using a spell
    slot of 4th level or higher, you can target one additional creature for each
    slot level above 3rd.
    """

    name = "Fly"
    level = 3
    casting_time = "1 action"
    casting_range = "Touch"
    components = ("V", "S", "M")
    materials = "A wing feather from any bird"
    duration = "Concentration, up to 10 minutes"
    ritual = False
    magic_school = "Transmutation"
    classes = ("Sorcerer", "Warlock", "Wizard")


class FogCloud(Spell):
    """You create a 20-foot-radius sphere of fog centered on a point within range. The
    sphere spreads around corners, and its area is heavily obscured, It lasts for
    the duration or until a wind of moderate or greater speed (at least 10 miles per
    hour) disperses it.

    **At Higher Levels:** When you cast this spell using a spell
    slot of 2nd level or higher, the radius of the fog increases by 20 feet for each
    slot level above 1st.
    """

    name = "Fog Cloud"
    level = 1
    casting_time = "1 action"
    casting_range = "120 feet"
    components = ("V", "S")
    materials = ""
    duration = "Concentration, up to 1 hour"
    ritual = False
    magic_school = "Conjuration"
    classes = ("Druid", "Ranger", "Sorcerer", "Wizard")


class Forbiddance(Spell):
    """You create a ward against magical travel that protects up to 40,000 square feet
    of floor space to a height of 30 feet above the floor. For the duration,
    creatures can't teleport into the area or use portals, such as those created by
    the gate spell, to enter the area. The spell proofs the area against planar
    travel, and therefore prevents creatures from accessing the area by way of the
    Astral Plane, Ethereal Plane, Feywild, Shadowfell, or the plane shift spell.

    In
    addition, the spell damages types of creatures that you choose when you cast
    it. Choose one or more of the following: celestials, elementals, fey, fiends,
    and undead. When a chosen creature enters the spell's area for the first time on
    a turn or starts its turn there, the creature takes 5d10 radiant or necrotic
    damage (your choice when you cast this spell).

    When you cast this spell, you
    can designate a password. A creature that speaks the password as it enters the
    area takes no damage from the spell.

    This spell's area can't overlap with the
    area of another forbiddance spell. If you cast forbiddance every day for 30 days
    in the same location, the spell lasts until it is dispelled, and the material
    components are consumed on the last casting.
    """

    name = "Forbiddance"
    level = 6
    casting_time = "10 minutes"
    casting_range = "Touch"
    components = ("V", "S", "M")
    materials = (
        "A sprinkling of holy water, rare incense, and powdered ruby worth at least"
        " 1,000 gp"
    )
    duration = "1 day"
    ritual = True
    magic_school = "Abjuration"
    classes = ("Cleric",)


class Forcecage(Spell):
    """An immobile, invisible, cube-shaped prison composed of magical force springs
    into existence around an area you choose within range. The prison can be a cage
    or a solid box as you choose.

    A prison in the shape of a cage can be up to 20
    feet on a side and is made from 1/2-inch diameter bars spaced 1/2 inch apart.

    A
    prison in the shape of a box can be up to 10 feet on a side, creating a solid
    barrier that prevents any matter from passing through it and blocking any spells
    cast into or out of the area.

    When you cast the spell, any creature that is
    completely inside the cage's area is trapped. Creatures only partially within
    the area, or those too large to fit inside the area, are pushed away from the
    center of the area until they are completely outside the area.

    A creature
    inside the cage can't leave it by nonmagical means. If the creature tries to use
    teleportation or interplanar travel to leave the cage, it must first make a
    Charisma saving throw. On a success, the creature can use that magic to exit the
    cage. On a failure, the creature can't exit the cage and wastes the use of the
    spell or effect. The cage also extends into the Ethereal Plane, blocking
    ethereal travel.
    This spell can't be dispelled by dispel magic.
    """

    name = "Forcecage"
    level = 7
    casting_time = "1 action"
    casting_range = "100 feet"
    components = ("V", "S", "M")
    materials = "Ruby dust worth 1,500 gp"
    duration = "1 hour"
    ritual = False
    magic_school = "Evocation"
    classes = ("Bard", "Warlock", "Wizard")


class Foresight(Spell):
    """You touch a willing creature and bestow a limited ability to see into the
    immediate future. For the duration, the target can't be surprised and has
    advantage on attack rolls, ability checks, and saving throws. Additionally,
    other creatures have disadvantage on attack rolls against the target for the
    duration.
    This spell immediately ends if you cast it again before its duration
    ends.
    """

    name = "Foresight"
    level = 9
    casting_time = "1 minute"
    casting_range = "Touch"
    components = ("V", "S", "M")
    materials = "A hummingbird feather"
    duration = "8 hours"
    ritual = False
    magic_school = "Divination"
    classes = ("Bard", "Druid", "Warlock", "Wizard")


class FreedomOfMovement(Spell):
    """You touch a willing creature. For the duration, the target's movement is
    unaffected by difficult terrain, and spells and other magical effects can
    neither reduce the target's speed nor cause the target to be paralyzed or
    restrained.

    The target can also spend 5 feet of movement to automatically
    escape from nonmagical restraints, such as manacles or a creature that has it
    grappled. Finally, being underwater imposes no penalties on the target's
    movement or attacks.
    """

    name = "Freedom Of Movement"
    level = 4
    casting_time = "1 action"
    casting_range = "Touch"
    components = ("V", "S", "M")
    materials = "A leather strap, bound around the arm or a similar appendage"
    duration = "Instantaneous"
    ritual = False
    magic_school = "Abjuration"
    classes = ("Bard", "Cleric", "Druid", "Ranger")


class Friends(Spell):
    """For the duration, you have advantage on all Charisma checks directed at one
    creature of your choice that isn't hostile toward you. When the spell ends, the
    creature realizes that you used magic to influence its mood and becomes hostile
    toward you. A creature prone to violence might attack you. Another creature
    might seek retribution in other ways (at the DM's discretion), depending on the
    nature of your interaction with it.
    """

    name = "Friends"
    level = 0
    casting_time = "1 action"
    casting_range = "Self"
    components = ("S", "M")
    materials = "A small amount of makeup applied to the face as this spell is cast"
    duration = "Concentration, up to 1 minute"
    ritual = False
    magic_school = "Enchantment"
    classes = ("Bard", "Sorcerer", "Warlock", "Wizard")


class Frostbite(Spell):
    """You cause numbing frost to form on one creature that you can see within range.
    The target must make a Constitution saving throw. On a failed save, the target
    takes 1d6 cold damage, and it has disadvantage on the next weapon attack roll it
    makes before the end of its next turn.
    The spell's damage increases by 1d6 when
    you reach 5th level (2d6), 11th level (3d6), and 17th level (4d6).
    """

    name = "Frostbite"
    level = 0
    casting_time = "1 action"
    casting_range = "60 feet"
    components = ("V", "S")
    materials = ""
    duration = "Instantaneous"
    ritual = False
    magic_school = "Evocation"
    classes = ("Druid", "Sorcerer", "Warlock", "Wizard")



from collections import defaultdict

from dungeonsheets import features, spells, weapons
from dungeonsheets.classes.classes import CharClass, SubClass


class ClericDomain(SubClass):
    name = "Generic Cleric Domain"
    _domain_spells = {1: [], 3: [], 5: [], 7: [], 9: []}

    @property
    def level(self):
        return self.owner.Cleric.level

    @property
    def spells_known(self):
        spells = []
        for lvl, sps in self._domain_spells.items():
            if self.level >= lvl:
                spells.extend(sps)
        return spells

    # All Domain spells are both known and prepared
    @property
    def spells_prepared(self):
        return self.spells_known


class KnowledgeDomain(ClericDomain):
    """The gods of knowledge-including Oghma, Boccob, Gilean, Aureon, and
    Thoth-value learning and understanding above all. Some teach that knowledge
    is to be gathered and shared in libraries and universities, or promote the
    practical knowledge of craft and invention. Some deities hoard knowledge
    and keep its secrets to themselves. And some promise their followers that
    they will gain tremendous power if they unlock the secrets of the
    multiverse. Followers of these gods study esoteric lore, collect old tomes,
    delve into the secret places of the earth, and learn all they can. Some
    gods of knowledge promote the practical knowledge of craft and invention,
    including smith deities like Gond, Reorx, Onatar, Moradin, Hephaestus, and
    Goibhniu.

    """

    name = "Knowledge Domain"
    _domain_spells = {
        1: [spells.Command, spells.Identify],
        3: [spells.Augury, spells.Suggestion],
        5: [spells.Nondetection, spells.SpeakWithDead],
        7: [spells.ArcaneEye, spells.Confusion],
        9: [spells.LegendLore, spells.Scrying],
    }
    features_by_level = defaultdict(list)
    features_by_level[1] = [features.BlessingsOfKnowledge]
    features_by_level[2] = [features.KnowledgeOfTheAncients]
    features_by_level[6] = [features.ReadThoughts]
    features_by_level[8] = [features.PotentSpellcasting]
    features_by_level[17] = [features.VisionsOfThePast]


class LifeDomain(ClericDomain):
    """The Life domain focuses on the vibrant positive energy-one of the
    fundamental forces of the universe- that sustains all life. The gods of
    life promote vitality and health through healing the sick and wounded,
    caring for those in need, and driving away the forces of death and
    undeath. Almost any non-evil deity can claim influence over this domain,
    particularly agricultural deities (such as Chauntea, Arawai, and Demeter),
    sun gods (such as Lathander, Pelor, and Re-Horakhty), gods of healing or
    endurance (such as Ilmater, Mishakal, Apollo, and Diancecht), and gods of
    home and community (such as Hestia, Hathor, and Boldrei).

    """

    name = "Life Domain"
    _domain_spells = {
        1: [spells.Bless, spells.CureWounds],
        3: [spells.LesserRestoration, spells.SpiritualWeapon],
        5: [spells.BeaconOfHope, spells.Revivify],
        7: [spells.DeathWard, spells.GuardianOfFaith],
        9: [spells.MassCureWounds, spells.RaiseDead],
    }
    _proficiencies_text = ("heavy armor",)
    features_by_level = defaultdict(list)
    features_by_level[1] = [features.DiscipleOfLife]
    features_by_level[2] = [features.PreserveLife]
    features_by_level[6] = [features.BlessedHealer]
    features_by_level[8] = [features.DivineStrikeLife]
    features_by_level[17] = [features.SupremeHealing]


class LightDomain(ClericDomain):
    """Gods of light-including Helm, Lathander, Pholtus, Branchala, the Silver
    Flame, Belenus, Apollo, and Re-Horakhty-promote the ideals of rebirth and
    renewal, truth, vigilance, and beauty, often using the symbol of the
    sun. Some of these gods are portrayed as the sun itself or as a charioteer
    who guides the sun across the sky. Others are tireless sentinels whose eyes
    pierce every shadow and see through every deception. Some are deities of
    beauty and artistry, who teach that art is a vehicle for the soul's
    improvement. Clerics of a god of light are enlightened souls infused with
    radiance and the power of their gods' discerning vision, charged with
    chasing away lies and burning away darkness.

    """

    name = "Light Domain"
    _domain_spells = {
        1: [spells.BurningHands, spells.FaerieFire],
        3: [spells.FlamingSphere, spells.ScorchingRay],
        5: [spells.Daylight, spells.Fireball],
        7: [spells.GuardianOfFaith, spells.WallOfFire],
        9: [spells.FlameStrike, spells.Scrying],
    }
    spells_known = (spells.Light,)
    spells_prepared = (spells.Light,)
    features_by_level = defaultdict(list)
    features_by_level[1] = [features.WardingFlare]
    features_by_level[2] = [features.RadianceOfTheDawn]
    features_by_level[6] = [features.ImprovedFlare]
    features_by_level[8] = [features.PotentSpellcasting]
    features_by_level[17] = [features.CoronaOfLight]


class NatureDomain(ClericDomain):
    """Gods of nature are as varied as the natural world itself, from inscrutable
    gods of the deep forests (such as Silvanus, Obad-Hai, Chislev, Balinor, and
    Pan) to friendly deities associated with particular springs and groves
    (such as Eldath). Druids revere nature as a whole and might serve one of
    these deities, practicing mysterious rites and reciting all-but-forgotten
    prayers in their own secret tongue. But many of these gods have clerics as
    well, champions who take a more active role in advancing the interests of
    a particular nature god. These clerics might hunt the evil monstrosities
    that despoil the woodlands, bless the harvest of the faithful, or wither
    the crops of those who anger their gods.

    """

    name = "Nature Domain"
    _domain_spells = {
        1: [spells.AnimalFriendship, spells.SpeakWithAnimals],
        3: [spells.Barkskin, spells.SpikeGrowth],
        5: [spells.PlantGrowth, spells.WindWall],
        7: [spells.DominateBeast, spells.GraspingVine],
        9: [spells.InsectPlague, spells.TreeStride],
    }
    _proficiencies_text = ("heavy armor",)
    features_by_level = defaultdict(list)
    features_by_level[1] = [features.AcolyteOfNature]
    features_by_level[2] = [features.CharmAnimalsAndPlants]
    features_by_level[6] = [features.DampenElements]
    features_by_level[8] = [features.DivineStrikeNature]
    features_by_level[17] = [features.MasterOfNature]


class TempestDomain(ClericDomain):
    """Gods whose portfolios include the Tempest domain - including Talos,
    Umberlee, Kord, Zeboim, the Devourer, Zeus, and Thor - govern storms, sea,
    and sky. They include gods of lightning and thunder, gods of earthquakes,
    some fire gods, and certain gods of violence, physical strength, and
    courage. In some pantheons, a god of this domain rules over other deities
    and is known for swift justice delivered by thunderbolts. In the pantheons
    of seafaring people, gods of this domain are ocean deities and the patrons
    of sailors. Tempest gods send their clerics to inspire fear in the common
    folk, either to keep those folk on the path of righteousness or to
    encourage them to offer sacrifices of propitiation to ward off divine
    wrath.

    """

    name = "Tempest Domain"
    _domain_spells = {
        1: [spells.FogCloud, spells.Thunderwave],
        3: [spells.GustOfWind, spells.Shatter],
        5: [spells.CallLightning, spells.SleetStorm],
        7: [spells.ControlWater, spells.IceStorm],
        9: [spells.DestructiveWave, spells.InsectPlague],
    }
    weapon_proficiencies = (weapons.MartialWeapon,)
    _proficiencies_text = ("martial weapons", "heavy armor")
    features_by_level = defaultdict(list)
    features_by_level[1] = [features.WrathOfTheStorm]
    features_by_level[2] = [features.DestructiveWrath]
    features_by_level[6] = [features.ThunderboltStrike]
    features_by_level[8] = [features.DivineStrikeTempest]
    features_by_level[17] = [features.Stormborn]


class TrickeryDomain(ClericDomain):
    """Gods of trickery-such as Tymora, Beshaba, Olidammara, the Traveler, Garl
    Glittergold, and Loki-are mischief-makers and instigators who stand as a
    constant challenge to the accepted order among both gods and
    mortals. They're patrons of thieves, scoundrels, gamblers, rebels, and
    liberators. Their clerics are a disruptive force in the world, puncturing
    pride, mocking tyrants, stealing from the rich, freeing captives, and
    flouting hollow traditions. They prefer subterfuge, pranks, deception, and
    theft rather than direct confrontation.

    """

    name = "Trickery Domain"
    _domain_spells = {
        1: [spells.CharmPerson, spells.DisguiseSelf],
        3: [spells.MirrorImage, spells.PassWithoutTrace],
        5: [spells.Blink, spells.DispelMagic],
        7: [spells.DimensionDoor, spells.Polymorph],
        9: [spells.DominatePerson, spells.ModifyMemory],
    }
    features_by_level = defaultdict(list)
    features_by_level[1] = [features.BlessingOfTheTrickster]
    features_by_level[2] = [features.InvokeDuplicity]
    features_by_level[6] = [features.CloakOfShadows]
    features_by_level[8] = [features.DivineStrikeTrickery]
    features_by_level[17] = [features.ImprovedDuplicity]


class WarDomain(ClericDomain):
    """War has many manifestations. It can make heroes of ordinary people. It can
    be desperate and horrific, with acts of cruelty and cowardice eclipsing
    instances of excellence and courage. In either case, the gods of war watch
    over warriors and reward them for their great deeds. The clerics of such
    gods excel in battle, inspiring others to fight the good fight or offering
    acts of violence as prayers. Gods of war include champions of honor and
    chivalry (such as Torm, Heironeous, and KiriJolith) as well as gods of
    destruction and pillage (such as Erythnul, the Fury, Gruumsh, and Ares) and
    gods of conquest and domination (such as Bane, Hextor, and
    Maglubiyet). Other war gods (such as Tempus, Nike, and Nuada) take a more
    neutral stance, promoting war in all its manifestations and supporting
    warriors in any circumstance.

    """

    name = "War Domain"
    _domain_spells = {
        1: [spells.DivineFavor, spells.ShieldOfFaith],
        3: [spells.MagicWeapon, spells.SpiritualWeapon],
        5: [spells.CrusadersMantle, spells.SpiritGuardians],
        7: [spells.FreedomOfMovement, spells.Stoneskin],
        9: [spells.FlameStrike, spells.HoldMonster],
    }
    weapon_proficiencies = (weapons.MartialWeapon,)
    _proficiencies_text = ("martial weapons", "heavy armor")
    features_by_level = defaultdict(list)
    features_by_level[1] = [features.WarPriest]
    features_by_level[2] = [features.GuidedStrike]
    features_by_level[6] = [features.WarGodsBlessing]
    features_by_level[8] = [features.DivineStrikeWar]
    features_by_level[17] = [features.AvatarOfBattle]


# SCAG
class ArcanaDomain(ClericDomain):
    """Magic is an energy that suffuses the multiverse and that fuels both
    destruction and creation. Gods of the Arcana domain know the secrets and
    potential of magic intimately. For some of these gods, magical knowledge
    is a great responsibility that comes with a special understanding of the
    nature of reality. Other gods of Arcana see magic as pure power, to be used
    as its wielder sees fit.

    The gods of this domain are often associated with knowledge, as learning
    and arcane power tend to go hand-in-hand. In the Realms, deities of this
    domain include Azuth and Mystra, as well as Corellon Larethian of the
    elven pantheon. In other worlds, this domain includes Hecate, Math
    Mathonwy, and Isis; the triple moon gods of Solinari , Lunitari, and
    Nuitari of Krynn; and Boccob, Vecna, and WeeJas of Greyhawk.

    """

    name = "Arcana Domain"
    _domain_spells = {
        1: [spells.DetectMagic, spells.MagicMissile],
        3: [spells.MagicWeapon, spells.NystulsMagicAura],
        5: [spells.DispelMagic, spells.MagicCircle],
        7: [spells.ArcaneEye, spells.LeomundsSecretChest],
        9: [spells.PlanarBinding, spells.TeleportationCircle],
    }
    features_by_level = defaultdict(list)
    features_by_level[1] = [features.ArcaneInitiate]
    features_by_level[2] = [features.ArcaneAbjuration]
    features_by_level[6] = [features.SpellBreaker]
    features_by_level[8] = [features.PotentSpellcasting]
    features_by_level[17] = [features.ArcaneMastery]


# XGTE
class ForgeDomain(ClericDomain):
    """The gods of the forge are patrons of artisans who work with metal, from a
    humble blacksmith who keeps a village in horseshoes and plow blades to the
    mighty elf artisan whose diamond-tipped arrows of mithral have felled demon
    lords. The gods of the forge teach that, with patience and hard work, even
    the most intractable metal can be transformed from a lump of ore to
    a beautifully wrought object. Clerics of these deities search for objects
    lost to the forces of darkness, liberate mines overrun by ores, and uncover
    rare and wondrous materials necessary to create potent magic items.
    Followers of these gods take great pride in their work, and they are willing
    to craft and use heavy armor and powerful weapons to protect them. Deities
    of this domain include Gond, Reorx, Onatar, Moradin, Hephaestus, and
    Goibhniu.

    """

    name = "Forge Domain"
    _domain_spells = {
        1: [spells.Identify, spells.SearingSmite],
        3: [spells.HeatMetal, spells.MagicWeapon],
        5: [spells.ElementalWeapon, spells.ProtectionFromEnergy],
        7: [spells.Fabricate, spells.WallOfFire],
        9: [spells.AnimateObjects, spells.Creation],
    }
    _proficiencies_text = ("heavy armor", "smith's tools")
    features_by_level = defaultdict(list)
    features_by_level[1] = [features.BlessingOfTheForge]
    features_by_level[2] = [features.ArtisansBlessing]
    features_by_level[6] = [features.SoulOfTheForge]
    features_by_level[8] = [features.DivineStrikeForge]
    features_by_level[17] = [features.SaintOfForgeAndFire]


class GraveDomain(ClericDomain):
    """Gods of the grave watch over the line between life and death. To these
    deities, death and the afterlife are a foundational part of the
    multiverse. To desecrate the peace of the dead is an abomination. Deities
    of the grave include Kelemvor, Wee jas, the ancestral spirits of the
    Undying Court, Hades, Anubis, and Osiris. Followers of these deities seek
    to put wandering spirits to rest, destroy the undead, and ease the
    suffering of the dying. Their magic also allows them to stave off death for
    a time. particularly for a person who still has some great work to
    accomplish in the world. This is a delay of death, not a denial of it, for
    death will eventually get its due.

    """

    name = "Grave Domain"
    _domain_spells = {
        1: [spells.Bane, spells.FalseLife],
        3: [spells.GentleRepose, spells.RayOfEnfeeblement],
        5: [spells.Revivify, spells.VampiricTouch],
        7: [spells.Blight, spells.DeathWard],
        9: [spells.AntilifeShell, spells.RaiseDead],
    }
    features_by_level = defaultdict(list)
    features_by_level[1] = [features.CircleOfMortality, features.EyesOfTheGrave]
    features_by_level[2] = [features.PathToTheGrave]
    features_by_level[6] = [features.SentinelAtDeathsDoor]
    features_by_level[8] = [features.PotentSpellcasting]
    features_by_level[17] = [features.KeeperOfSouls]


class DeathDomain(ClericDomain):
    """The Death domain is concerned with the forces that cause death, as
    well as the negative energy that gives rise to undead creatures.
    Deities such as Chemosh, Myrkul, and Wee Jas are patrons of necromancers,
    death knights, liches, mummy lords, and vampires. Gods of the Death
    domain also embody murder (Anubis, Bhaal, and Pyremius), pain
    (Iuz or Loviatar), disease or poison (Incabulos, Talona, or Morgion),
    and the underworld (Hades and Hel)..

    """

    name = "Death Domain"
    _domain_spells = {
        1: [spells.FalseLife, spells.RayOfSickness],
        3: [spells.BlindnessDeafness, spells.RayOfEnfeeblement],
        5: [spells.AnimateDead, spells.VampiricTouch],
        7: [spells.Blight, spells.DeathWard],
        9: [spells.AntilifeShell, spells.Cloudkill],
    }
    weapon_proficiencies = (weapons.MartialWeapon,)
    _proficiencies_text = ("martial weapons",)
    features_by_level = defaultdict(list)
    features_by_level[1] = [features.Reaper]
    features_by_level[2] = [features.TouchOfDeathCleric]  # a different one though
    features_by_level[6] = [features.InescapableDestruction]
    features_by_level[8] = [features.DivineStrikeDeath]
    features_by_level[17] = [features.ImprovedReaper]


class Cleric(CharClass):
    name = "Cleric"
    hit_dice_faces = 8
    subclass_select_level = 1
    saving_throw_proficiencies = ("wisdom", "charisma")
    primary_abilities = ("wisdom",)
    _proficiencies_text = (
        "light armor",
        "medium armor",
        "shields",
        "all simple weapons",
    )
    weapon_proficiencies = (weapons.SimpleWeapon,)
    multiclass_weapon_proficiencies = ()
    _multiclass_proficiencies_text = ("light armor", "medium armor", "shields")
    class_skill_choices = ("History", "Insight", "Medicine", "Persuasion", "Religion")
    features_by_level = defaultdict(list)
    features_by_level[2] = (
        features.ChannelDivinity,
        features.TurnUndead,
    )
    features_by_level[5] = (features.DestroyUndead,)
    features_by_level[10] = (features.DivineIntervention,)
    subclasses_available = (
        KnowledgeDomain,
        LifeDomain,
        LightDomain,
        NatureDomain,
        TempestDomain,
        TrickeryDomain,
        WarDomain,
        ArcanaDomain,
        ForgeDomain,
        GraveDomain,
        DeathDomain,
    )
    spellcasting_ability = "wisdom"
    spell_slots_by_level = {
        # char_lvl: (cantrips, 1st, 2nd, 3rd, ...)
        1: (3, 2, 0, 0, 0, 0, 0, 0, 0, 0),
        2: (3, 3, 0, 0, 0, 0, 0, 0, 0, 0),
        3: (3, 4, 2, 0, 0, 0, 0, 0, 0, 0),
        4: (4, 4, 3, 0, 0, 0, 0, 0, 0, 0),
        5: (4, 4, 3, 2, 0, 0, 0, 0, 0, 0),
        6: (4, 4, 3, 3, 0, 0, 0, 0, 0, 0),
        7: (4, 4, 3, 3, 1, 0, 0, 0, 0, 0),
        8: (4, 4, 3, 3, 2, 0, 0, 0, 0, 0),
        9: (4, 4, 3, 3, 3, 1, 0, 0, 0, 0),
        10: (5, 4, 3, 3, 3, 2, 0, 0, 0, 0),
        11: (5, 4, 3, 3, 3, 2, 1, 0, 0, 0),
        12: (5, 4, 3, 3, 3, 2, 1, 0, 0, 0),
        13: (5, 4, 3, 3, 3, 2, 1, 1, 0, 0),
        14: (5, 4, 3, 3, 3, 2, 1, 1, 0, 0),
        15: (5, 4, 3, 3, 3, 2, 1, 1, 1, 0),
        16: (5, 4, 3, 3, 3, 2, 1, 1, 1, 0),
        17: (5, 4, 3, 3, 3, 2, 1, 1, 1, 1),
        18: (5, 4, 3, 3, 3, 3, 1, 1, 1, 1),
        19: (5, 4, 3, 3, 3, 3, 2, 1, 1, 1),
        20: (5, 4, 3, 3, 3, 3, 2, 2, 1, 1),
    }



from dungeonsheets.spells.spells import Spell


class GaseousForm(Spell):
    """You transform a willing creature you touch, along with everything it's wearing
    and carrying, into a misty cloud for the duration. The spell ends if the
    creature drops to 0 hit points. An incorporeal creature isn't affected.

    While in this form, the target's only method of movement is a
    flying speed of 10 feet.  The target can enter and occupy the
    space of another creature. The target has resistance to nonmagical
    damage, and it has advantage on Strength, Dexterity, and
    Constitution saving throws. The target can pass through small
    holes, narrow openings, and even mere cracks, though it treats
    liquids as though they were solid surfaces. The target can't fall
    and remains hovering in the air even when stunned or otherwise
    incapacitated.

    While in the form of a misty cloud, the target can't talk or
    manipulate objects, and any objects it was carrying or holding
    can't be dropped, used, or otherwise interacted with. The target
    can't attack or cast spells.

    """

    name = "Gaseous Form"
    level = 3
    casting_time = "1 action"
    casting_range = "Touch"
    components = ("V", "S", "M")
    materials = "A bit of gauze and a wisp of smoke"
    duration = "Concentration, up to 1 hour"
    ritual = False
    magic_school = "Transmutation"
    classes = ("Sorcerer", "Warlock", "Wizard")


class Gate(Spell):
    """You conjure a portal linking an unoccupied space you can see within
    range to a precise location on a different plane of existence. The
    portal is a circular opening, which you can make 5 to 20 feet in
    diameter. You can orient the portal in any direction you
    choose. The portal lasts for the duration.

    The portal has a front and a back on each plane where it
    appears. Travel through the portal is possible only by moving
    through its front. Anything that does so is instantly transported
    to the other plane, appearing in the unoccupied space nearest to
    the portal.

    Deities and other planar rulers can prevent portals created by
    this spell from opening in their presence or anywhere within their
    domains.

    When you cast this spell, you can speak the name of a specific
    creature (a pseudonym, title, or nickname doesn't work). If that
    creature is on a plane other than the one you are on, the portal
    opens in the named creature's immediate vicinity and draws the
    creature through it to the nearest unoccupied space on your side
    of the portal. You gain no special power over the creature, and it
    is free to act as the Dm deems appropriate. It might leave, attack
    you, or help you.

    """

    name = "Gate"
    level = 9
    casting_time = "1 action"
    casting_range = "60 feet"
    components = ("V", "S", "M")
    materials = "A diamond worth at least 5,000 gp"
    duration = "Concentration, up to 1 minute"
    ritual = False
    magic_school = "Conjuration"
    classes = ("Cleric", "Sorcerer", "Wizard")


class Geas(Spell):
    """You place a magical command on a creature that you can see within
    range, forcing it to carry out some service or refrain from some
    action or course of actiity as you decide.

    If the creature can understand you, it must succeed on a Wisdom
    saving throw or become charmed by you for the duration. While the
    creature is charmed by you, it takes 5d10 psychic damage each time
    it acts in a manner directly counter to your instructions, but no
    more than once each day. A creature that can't understand you is
    unaffected by the spell.

    You can issue any command you choose, short of an activity that
    would result in certain death. Should you issue a suicidal
    command, the spell ends. You can end the spell early by using an
    action to dismiss it. A remove curse, greater restoration, or wish
    spell also ends it.

    **At Higher Levels:** When you cast this spell usinga spell slot of
    7th or 8th level, the duration is 1 year.  When you cast this
    spell using a spell slot of 9th level, the spell lasts until it is
    ended by one of the spells mentioned above.

    """

    name = "Geas"
    level = 5
    casting_time = "1 minute"
    casting_range = "60 feet"
    components = ("V",)
    materials = ""
    duration = "30 days"
    ritual = False
    magic_school = "Enchantment"
    classes = ("Bard", "Cleric", "Druid", "Paladin", "Wizard")


class GentleRepose(Spell):
    """You touch a corpse or other remains. For the duration, the target
    is protected from decay and can't become undead.

    The spell also effectively extends the time limit on raising the
    target from the dead, since days spent under the influence of this
    spell don't count against the time limit of spells such as raise
    dead.

    """

    name = "Gentle Repose"
    level = 2
    casting_time = "1 action"
    casting_range = "Touch"
    components = ("V", "S", "M")
    materials = (
        "A pinch of salt and one copper piece placed on each of the corpse's eyes,"
        " which must remain there for the duration"
    )
    duration = "10 days"
    ritual = True
    magic_school = "Necromancy"
    classes = ("Cleric", "Wizard")


class GiantInsect(Spell):
    """You transform up to ten centipedes, three spiders, five wasps, or
    one scorpion within range into giant versions of their natural
    forms for the duration. A centipede becomes a giant centipede, a
    spider becaomes a giant spider, a wasp becomes a giant wasp, and a
    scorpion becomes a giant scorpion.

    Each creature obeys your verbal commands, and in combat, they act
    on your turn each round. The DM has the statistics for these
    creatures and resolves their actions and movement.

    A creature remains in its giant size for the duration, until it
    drops to 0 hit points, or until you use an action to dismiss the
    effect on it.

    The DM might allow you to choose different targets. For example,
    if you transform a bee, its giant version might have the same
    statistics as a giant wasp.

    """

    name = "Giant Insect"
    level = 4
    casting_time = "1 action"
    casting_range = "30 feet"
    components = ("V", "S")
    materials = ""
    duration = "Concentration, up to 10 minutes"
    ritual = False
    magic_school = "Transmutation"
    classes = ("Druid",)


class Glibness(Spell):
    """Until the spell ends, when you make a Charisma check, you can
    replace the number you roll with a 15. Additionally, no matter
    what you say, magic that would determine if you are telling the
    truth indicates that you are being truthful.

    """

    name = "Glibness"
    level = 8
    casting_time = "1 action"
    casting_range = "Self"
    components = ("V",)
    materials = ""
    duration = "1 hour"
    ritual = False
    magic_school = "Transmutation"
    classes = ("Bard", "Warlock")


class GlobeOfInvulnerability(Spell):
    """An immobile, faintly shimmering barrier springs into existence in a
    10-foot radius around you and remains for the duration.

    Any spell of 5th level or lower cast from outside the barrier
    can't affect creatures or objects within it, even if the spell is
    cast using a higher level spell slot. Such a spell can target
    creatures and objects within the barrier, but the spell has no
    effect on them.  Similarly, the area within the barrier is
    excluded from the areas affected by such spells.

    **At Higher Levels:** When you cast this spell using a spell slot
    of 7th level or higher, the barrier blocks spells of one level
    higher for each slot level above 6th.

    """

    name = "Globe Of Invulnerability"
    level = 6
    casting_time = "1 action"
    casting_range = "Self (10-foot radius)"
    components = ("V", "S", "M")
    materials = "A glass or crystal bead that shatters when the spell ends"
    duration = "Concentration, up to 1 minute"
    ritual = False
    magic_school = "Abjuration"
    classes = ("Sorcerer", "Wizard")


class GlyphOfWarding(Spell):
    """When you cast this spell, you inscribe a glyph that harms other
    creatures, either upon a surface (such as a table or a section of
    floor or wall) or within an object that can be closed (such as a
    book, a scroll, or a treasure chest) to conceal the glyph.

    If you choose a surface, the glyph can cover an area of the
    surface no larger than 10 feet in diameter. If you choose an
    object, that object must remain in its place; if the object is
    moved more than 10 feet from where you cast this spell, the glyph
    is broken, and the spell ends without being triggered.

    The glyph is nearly invisible and requires a successful
    Intelligence (Investigation) check against your spell save DC to
    be found.

    You decide what triggers the glyph when you cast the spell. For
    glyphs inscribed on a surface, the most typical triggers include
    touching or standing on the glyph, removing another object
    covering the glyph, approaching within a certain distance of the
    glyph, or manipulating the object on which the glyph is
    inscribed. For glyphs inscribed within an object, the most common
    triggers include opening that object, approaching within a certain
    distance of the object, or seeing or reading the glyph. Once a
    glyph is triggered, this spell ends.

    You can further refine the trigger so the spell activates only
    under certain circumstances or according to physical
    characteristics (such as height or weight), creature kind (for
    example, the ward could be set to affect aberrations or drow), or
    alignment. You can also set conditions for creatures that don't
    trigger the glyph, such as those who say a certain password.

    When you inscribe the glyph, choose explosive runes or a spell
    glyph.

    **Explosive Runes**
    When triggered, the glyph erupts with magical energy in a
    20-foot-radius sphere centered on the glyph. The sphere spreads
    around corners. Each creature in the area must make a Dexterity
    saving throw. A creature takes 5d8 acid, cold, fire, lightning, or
    thunder damage on a failed saving throw (your choice when you
    create the glyph), or half as much damage on a successful one.

    **Spell Glyph**
    You can store a prepared spell of 3rd level or lower in the glyph
    by casting it as part of creating the glyph. The spell must target
    a single creature or an area. The spell being stored has no
    immediate effect when cast in this way. When the glyph is
    triggered, the stored spell is cast. If the spell has a target, it
    targets the creature that triggered the glyph. If the spell
    affects an area, the area is centered on that creature. If the
    spell summons hostile creatures or creates harmful objects or
    traps, they appear as close as possible to the intruder and attack
    it. If the spell requires concentration, it lasts until the end of
    its full duration.

    **At Higher Levels:** When you cast this spell using a spell slot
    of 4th Level or higher, the damage of an explosive runes glyph
    increases by 1d8 for each slot level above 3rd. If you create a
    spell glyph, you can store any spell of up to the same level as
    the slot you use for the glyph of warding.

    """

    name = "Glyph Of Warding"
    level = 3
    casting_time = "1 hour"
    casting_range = "Touch"
    components = ("V", "S", "M")
    materials = (
        "Incense and powdered diamond worth at least 200 gp, which the spell consumes"
    )
    duration = "Until dispelled or triggered"
    ritual = False
    magic_school = "Abjuration"
    classes = (
        "you",
        "cast",
        "this",
        "spell",
        "using",
        "a",
        "spell",
        "slot",
        "of",
        "4th",
        "level",
        "or",
        "higher",
        "the",
        "damage",
        "of",
        "an",
        "explosive",
        "runes",
        "glyph",
        "increases",
        "by",
        "1d8",
        "for",
        "each",
        "slot",
        "level",
        "above",
        "3rd.",
        "If",
        "you",
        "create",
        "a",
        "spell",
        "glyph",
        "you",
        "can",
        "store",
        "any",
        "spell",
        "of",
        "up",
        "to",
        "the",
        "same",
        "level",
        "as",
        "the",
        "slot",
        "you",
        "use",
        "for",
        "the",
        "glyph",
        "of",
    )


class Goodberry(Spell):
    """Up to ten berries appear in your hand and are infused with magic for the
    duration. A creature can use its action to eat one berry. Eating a berry
    restores 1 hit point, and the berry provides enough nourishment to sustain a
    creature for one day.
    The berries lose their potency if they have not been
    consumed within 24 hours of the casting of this spell.
    """

    name = "Goodberry"
    level = 1
    casting_time = "1 action"
    casting_range = "Touch"
    components = ("V", "S", "M")
    materials = "A sprig of mistletoe"
    duration = "Instantaneous"
    ritual = False
    magic_school = "Transmutation"
    classes = ("Druid", "Ranger")


class GraspingVine(Spell):
    """You conjure a vine that sprouts from the ground in an unoccupied space of your
    choice that you can see within range. When you cast this spell, you can direct
    the vine to lash out at a creature within 30 feet of it that you can see. That
    creature must succeed on a Dexterity saving throw or be pulled 20 feet directly
    toward the vine.

    Until the spell ends, you can direct the vine to lash out at
    the same creature or another one as a bonus action on each of your turns.
    """

    name = "Grasping Vine"
    level = 4
    casting_time = "1 bonus action"
    casting_range = "30 feet"
    components = ("V", "S")
    materials = ""
    duration = "Concentration, up to 1 minute"
    ritual = False
    magic_school = "Conjuration"
    classes = ("Druid", "Ranger")


class Grease(Spell):
    """Slick grease covers the ground in a 10-foot square centered on a point within
    range and turns it into difficult terrain for the duration.

    When the grease
    appears, each creature standing in its area must succeed on a Dexterity saving
    throw or fall prone. A creature that enters the area or ends its turn there must
    also succeed on a Dexterity saving throw or fall prone.
    """

    name = "Grease"
    level = 1
    casting_time = "1 action"
    casting_range = "60 feet"
    components = ("V", "S", "M")
    materials = "A bit of pork rind or butter"
    duration = "1 minute"
    ritual = False
    magic_school = "Conjuration"
    classes = ("Wizard",)


class GreaterInvisibility(Spell):
    """You or a creature you touch becomes invisible until the spell
    ends. Anything the target is wearing or carrying is invisible as
    long as it is on the target's person.

    """

    name = "Greater Invisibility"
    level = 4
    casting_time = "1 action"
    casting_range = "Touch"
    components = ("V", "S")
    materials = ""
    duration = "Concentration, up to 1 minute"
    ritual = False
    magic_school = "Illusion"
    classes = ("Bard", "Sorcerer", "Wizard")


class GreaterRestoration(Spell):
    """You imbue a creature you touch with positive energy to undo a
    debilitating effect. You can reduce the target's exhaustion level
    by one, or end one of the following effects on the target:

    - One effect that charmed or petrified the target
    - One curse, including the target's attunement to a cursed magic item
    - Any reduction to one of the target's ability scores
    - One effect reducing the target's hit point maximum

    """

    name = "Greater Restoration"
    level = 5
    casting_time = "1 action"
    casting_range = "Touch"
    components = ("V", "S", "M")
    materials = "Diamond dust worth at least 100 gp, which the spell consumes"
    duration = "Instantaneous"
    ritual = False
    magic_school = "Abjuration"
    classes = ("Bard", "Cleric", "Druid")


class GreenFlameBlade(Spell):
    """As part of the action used to cast this spell, you must make a melee attack with
    a weapon against one creature within the spell's range, otherwise the spell
    fails. On a hit, the target suffers the attack's normal effects, and green fire
    leaps from the target to a different creature of your choice that you can see
    within 5 feet of it. The second creature takes fire damage equal to your
    spellcasting ability modifier. This spell's damage increases when you reach
    higher levels.

    **At Higher Levels:** At 5th level, the melee attack deals an extra
    1d8 fire damage to the target, and the fire damage to the second creature
    increases to 1d8 + your spellcasting ability modifier. Both damage rolls
    increase by 1d8 at 11th level and 17th level.
    """

    name = "Green-Flame Blade"
    level = 0
    casting_time = "1 action"
    casting_range = "5 feet"
    components = ("V", "M")
    materials = "A weapon"
    duration = "Instantaneous"
    ritual = False
    magic_school = "Evocation"
    classes = ("Sorcerer", "Warlock", "Wizard")


class GuardianOfFaith(Spell):
    """A Large spectral guardian appears and hovers for the duration in an unoccupied
    space of your choice that you can see within range. The guardian occupies that
    space and is indistinct except for a gleaming sword and shield emblazoned with
    the symbol of your deity.

    Any creature hostile to you that moves to a space
    within 10 feet of the guardian for the firs time on a turn must succeed on a
    Dexterity saving throw. The creature takes 20 radiant damage on a failed save,
    or half as much damage on a successful one. The guardian vanishes when it has
    dealt a total of 60 damage.
    """

    name = "Guardian Of Faith"
    level = 4
    casting_time = "1 action"
    casting_range = "30 feet"
    components = ("V",)
    materials = ""
    duration = "8 hours"
    ritual = False
    magic_school = "Conjuration"
    classes = ("Cleric",)


class GuardianOfNature(Spell):
    """A nature spirit answers your call and transforms you into a
    powerful guardian. The transformation lasts until the spell
    ends. You choose one of the following forms to assume: Primal
    Beast or Great Tree.

    **Primal Beast.** Bestial fur covers your body, your facial
    features become feral, and you gain the following benefits:

    - Your walking speed increases by 10 feet.
    - You gain darkvision with a range of 120 feet.
    - You make Strength-based attack rolls with advantage.
    - Your melee weapon attacks deal an extra 1d6 force damage on a
      hit.

    **Great Tree.** Your skin appears barky, leaves sprout from your
    hair, and you gain the following benefits:

    - You gain 10 temporary hit points.
    - You make Constitution saving throws with advantage.
    - You make Dexterity- and Wisdom-based attack rolls with
      advantage.
    - While you are on the ground, the ground within 15 feet of you is
      difficult terrain for your enemies.

    """

    name = "Guardian Of Nature"
    level = 4
    casting_time = "1 bonus action"
    casting_range = "Self"
    components = ("V",)
    materials = ""
    duration = "Concentration, up to 1 minute"
    ritual = False
    magic_school = "Transmutation"
    classes = ("Druid", "Ranger")


class GuardsAndWards(Spell):
    """You create a ward that protects up to 2,500 square feet of floor
    space (an area 50 feet square, or one hundred 5-foot squares or
    twenty-five 10-foot squares). The warded area can be up to 20 feet
    tall, and shaped as you desire. You can ward several stories of a
    stronghold by dividing the area among them, as long as you can
    walk into each contiguous area while you are casting the spell.

    When you cast this spell, you can specify individuals that are
    unaffected by any or all of the effects that you choose. You can
    also specify a password that, when spoken aloud, makes the speaker
    immune to these effects.

    Guards and wards creates the following effects within the warded
    area.

    - **Corridors.** Fog fills all the warded corridors, making them
      heavily obscured. In addition, at each intersection or branching
      passage offering a choice of direction, there is a 50 percent
      chance that a creature other than you will believe it is going
      in the opposite direction from the one it chooses.
    - **Doors.** All doors in the warded area are magically locked, as
      if sealed by an arcane lock spell. In addition, you can cover up
      to ten doors with an illusion (equivalent to the illusory object
      function of the m inor illusion spell) to make them appear as
      plain sections of wall.
    - **Stairs.** Webs fill all stairs in the warded area from top to
      bottom, as the web spell. These strands regrow in 10 minutes if
      they are burned or torn away while guards and wards lasts.
    - **Other Spell Effect.** You can place your choice of one of the
      following magical effects within the warded area of the
      stronghold.

      - Place dancing lights in four corridors. You can designate a
        simple program that the lights repeat as long as guards and
        wards lasts.
      - Place magic mouth in two locations.
      - Place stinking cloud in two locations. The vapors appear in
        the places you designate; they return within 10 minutes if
        dispersed by wind while guards and wards lasts.
      - Place a constant gust of wind in one corridor or room.
      - Place a suggestion in one location. You select an area of up
        to 5 feet square, and any creature that enters or passes
        through the area receives the suggestion mentally.

    The whole warded area radiates magic. A dispel magic cast on a
    specific effect, if successful, removes only that effect.  You can
    create a permanently guarded and warded structure by casting this
    spell there every day for one year.

    """

    name = "Guards And Wards"
    level = 6
    casting_time = "10 minutes"
    casting_range = "Touch"
    components = ("V", "S", "M")
    materials = (
        "Burning incense, a small measure of brimstone and "
        "oil, a knotted string, a small amount of umber hulk blood, and "
        "a small silver rod worth at least 10 gp"
    )
    duration = "24 hours"
    ritual = False
    magic_school = "Abjuration"
    classes = ("Bard", "Wizard")


class Guidance(Spell):
    """You touch one willing creature. Once before the spell ends, the target can roll
    a d4 and add the number rolled to one ability check of its choice. It can roll
    the die before or after making the ability check. The spell then ends.
    """

    name = "Guidance"
    level = 0
    casting_time = "1 action"
    casting_range = "Touch"
    components = ("V", "S")
    materials = ""
    duration = "Concentration, up to 1 minute"
    ritual = False
    magic_school = "Divination"
    classes = ("Cleric", "Druid")


class GuidingBolt(Spell):
    """A flash of light streaks toward a creature of your choice within
    range.  Make a ranged spell attack against the target. On a hit,
    the target takes 4d6 radiant damage, and the next attack roll made
    against this target before the end of your next turn has
    advantage, thanks to the mystical dim light glittering on the
    target until then.

    **At Higher Levels:** When you cast this spell using a spell slot
    of 2nd level or higher, the damage increases by 1d6 for each slot
    level above 1st.

    """

    name = "Guiding Bolt"
    level = 1
    casting_time = "1 action"
    casting_range = "120 feet"
    components = ("V", "S")
    materials = ""
    duration = "1 round"
    ritual = False
    magic_school = "Evocation"
    classes = ("Cleric",)


class GuidingHand(Spell):
    """You create a Tiny incorporeal hand of shimmering light in an
    unoccupied space you can see within range. The hand exists for the
    duration, but it disappears if you teleport or you travel to a
    different plane of existence.

    When the hand appears, you name one major landmark, such as a
    city, mountain, castle, or battlefield on the same plane of
    existence as you.  Someone in history must have visited the site
    and mapped it. If the landmark appears on no map in existence, the
    spell fails. Otherwise, whenever you move toward the hand, it
    moves away from you at the same speed you moved, and it moves in
    the direction of the landmark, always remaining 5 feet away from
    you.

    If you don't move toward the hand, it remains in place until you
    do and beckons for you to follow once every 1d4 minutes.

    """

    name = "Guiding Hand"
    level = 1
    casting_time = "1 minute"
    casting_range = "5 feet"
    components = ("V", "S")
    materials = ""
    duration = "Concentration, up to 8 hours"
    ritual = True
    magic_scool = "Divination"
    classes = ("Bard", "Cleric", "Druid", "Wizard")


class Gust(Spell):
    """You seize the air and compel it to create one of the following
    effects at a point you can see within range:

    - One Medium or smaller creature that you choose must succeed on a
      Strength saving throw or be pushed up to 5 feet away from you.
    - You create a small blast of air capable of moving one object
      that is neither held nor carried and that weighs no more than 5
      pounds. The object is pushed up to 10 feet away from you. It
      isn't pushed with enough force to cause damage.
    - You create a harmless sensory affect using air, such as causing
      leaves to rustle, wind to slam shutters shut, or your clothing
      to ripple in a breeze.

    """

    name = "Gust"
    level = 0
    casting_time = "1 action"
    casting_range = "30 feet"
    components = ("V", "S")
    materials = ""
    duration = "Instantaneous"
    ritual = False
    magic_school = "Transmutation"
    classes = ("Druid", "Sorcerer", "Wizard")


class GustOfWind(Spell):
    """A line of strong wind 60 feet long and 10 feet wide blasts from you
    in a direction you choose for the spell's duration. Each creature
    that starts its turn in the line must succeed on a Strength saving
    throw or be pushed 15 feet away from you in a direction following
    the line.

    Any creature in the line must spend 2 feet of movement for every 1
    foot it moves when moving closer to you.

    The gust disperses gas or vapor, and it extinguishes candles,
    torches, and similar unprotected flames in the area. It causes
    protected flames, such as those of lanterns, to dance wildly and
    has a 50 percent chance to extinguish them.

    As a bonus action on each of your turns before the spell ends, you
    can change the direction in which the line blasts from you.

    """

    name = "Gust Of Wind"
    level = 2
    casting_time = "1 action"
    casting_range = "Self (60-foot line)"
    components = ("V", "S", "M")
    materials = "A legume seed"
    duration = "Concentration, up to 1 minute"
    ritual = False
    magic_school = "Evocation"
    classes = ("Druid", "Sorcerer", "Wizard")



"""
A collection of monsters from D&D 5e's SRD

This file was autogenerated by https://github.com/stravajiaxen/5e-srd-to-py
"""

from dungeonsheets.monsters.monsters import Monster
from dungeonsheets.stats import Ability


class Gargoyle(Monster):
    """
    False Appearance.
      While the gargoyle remains motion less, it is indistinguishable from
      an inanimate statue.
    Multiattack.
      The gargoyle makes two attacks: one with its bite and one with its
      claws.
    Bite.
      Melee Weapon Attack: +4 to hit, reach 5 ft., one target. Hit: 5 (1d6 +
      2) piercing damage.
    Claws.
      Melee Weapon Attack: +4 to hit, reach 5 ft., one target. Hit: 5 (1d6 +
      2) slashing damage.
    """
    name = "Gargoyle"
    description = "Medium elemental, chaotic evil"
    challenge_rating = 2
    armor_class = 15
    skills = ""
    senses = "Darkvision 60 ft., Passive Perception 10"
    languages = "Terran"
    strength = Ability(15)
    dexterity = Ability(11)
    constitution = Ability(16)
    intelligence = Ability(6)
    wisdom = Ability(11)
    charisma = Ability(7)
    speed = 30
    swim_speed = 0
    fly_speed = 60
    climb_speed = 0
    hp_max = 52
    hit_dice = "7d8"
    spells = []


class GelatinousCube(Monster):
    """
    Ooze Cube.
      The cube takes up its entire space. Other creatures can enter the
      space, but a creature that does so is subjected to the cube's Engulf
      and has disadvantage on the saving throw.
      
      Creatures inside the cube can be seen but have total cover.
      
      A creature within 5 feet of the cube can take an action to pull a
      creature or object out of the cube. Doing so requires a successful DC
      12 Strength check, and the creature making the attempt takes 10 (3d6)
      acid damage.
      
      The cube can hold only one Large creature or up to four Medium or
      smaller creatures inside it at a time.
    Transparent.
      Even when the cube is in plain sight, it takes a successful DC 15
      Wisdom (Perception) check to spot a cube that has neither moved nor
      attacked. A creature that tries to enter the cube's space while
      unaware of the cube is surprised by the cube.
    Pseudopod.
      Melee Weapon Attack: +4 to hit, reach 5 ft., one creature. Hit: 10
      (3d6) acid damage.
    Engulf.
      The cube moves up to its speed. While doing so, it can enter Large or
      smaller creatures' spaces. Whenever the cube enters a creature's
      space, the creature must make a DC 12 Dexterity saving throw.
      
      On a successful save, the creature can choose to be pushed 5 feet back
      or to the side of the cube. A creature that chooses not to be pushed
      suffers the consequences of a failed saving throw.
      
      On a failed save, the cube enters the creature's space, and the
      creature takes 10 (3d6) acid damage and is engulfed. The engulfed
      creature can't breathe, is restrained, and takes 21 (6d6) acid damage
      at the start of each of the cube's turns. When the cube moves, the
      engulfed creature moves with it.
      
      An engulfed creature can try to escape by taking an action to make a
      DC 12 Strength check. On a success, the creature escapes and enters a
      space of its choice within 5 feet of the cube.
    """
    name = "Gelatinous Cube"
    description = "Large ooze, unaligned"
    challenge_rating = 2
    armor_class = 6
    skills = ""
    senses = "Blindsight 60 ft. (blind beyond this radius), Passive Perception 8"
    languages = ""
    strength = Ability(14)
    dexterity = Ability(3)
    constitution = Ability(20)
    intelligence = Ability(1)
    wisdom = Ability(6)
    charisma = Ability(1)
    speed = 15
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 84
    hit_dice = "8d10"
    spells = []


class Ghast(Monster):
    """
    Stench.
      Any creature that starts its turn within 5 ft. of the ghast must
      succeed on a DC 10 Constitution saving throw or be poisoned until the
      start of its next turn. On a successful saving throw, the creature is
      immune to the ghast's Stench for 24 hours.
    Turn Defiance.
      The ghast and any ghouls within 30 ft. of it have advantage on saving
      throws against effects that turn undead.
    Bite.
      Melee Weapon Attack: +3 to hit, reach 5 ft., one creature. Hit: 12
      (2d8 + 3) piercing damage.
    Claws.
      Melee Weapon Attack: +5 to hit, reach 5 ft., one target. Hit: 10 (2d6
      + 3) slashing damage. If the target is a creature other than an
      undead, it must succeed on a DC 10 Constitution saving throw or be
      paralyzed for 1 minute. The target can repeat the saving throw at the
      end of each of its turns, ending the effect on itself on a success.
    """
    name = "Ghast"
    description = "Medium undead, chaotic evil"
    challenge_rating = 2
    armor_class = 13
    skills = ""
    senses = "Darkvision 60 ft., Passive Perception 10"
    languages = "Common"
    strength = Ability(16)
    dexterity = Ability(17)
    constitution = Ability(10)
    intelligence = Ability(11)
    wisdom = Ability(10)
    charisma = Ability(8)
    speed = 30
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 36
    hit_dice = "8d8"
    spells = []


class Ghost(Monster):
    """
    Ethereal Sight.
      The ghost can see 60 ft. into the Ethereal Plane when it is on the
      Material Plane, and vice versa.
    Incorporeal Movement.
      The ghost can move through other creatures and objects as if they were
      difficult terrain. It takes 5 (1d10) force damage if it ends its turn
      inside an object.
    Withering Touch.
      Melee Weapon Attack: +5 to hit, reach 5 ft., one target. Hit: 17 (4d6
      + 3) necrotic damage.
    Etherealness.
      The ghost enters the Ethereal Plane from the Material Plane, or vice
      versa. It is visible on the Material Plane while it is in the Border
      Ethereal, and vice versa, yet it can't affect or be affected by
      anything on the other plane.
    Horrifying Visage.
      Each non-undead creature within 60 ft. of the ghost that can see it
      must succeed on a DC 13 Wisdom saving throw or be frightened for 1
      minute. If the save fails by 5 or more, the target also ages 1d4 + 10
      years. A frightened target can repeat the saving throw at the end of
      each of its turns, ending the frightened condition on itself on a
      success. If a target's saving throw is successful or the effect ends
      for it, the target is immune to this ghost's Horrifying Visage for the
      next 24 hours. The aging effect can be reversed with a greater
      restoration spell, but only within 24 hours of it occurring.
    Possession (Recharge 6).
      One humanoid that the ghost can see within 5 ft. of it must succeed on
      a DC 13 Charisma saving throw or be possessed by the ghost; the ghost
      then disappears, and the target is incapacitated and loses control of
      its body. The ghost now controls the body but doesn't deprive the
      target of awareness. The ghost can't be targeted by any attack, spell,
      or other effect, except ones that turn undead, and it retains its
      alignment, Intelligence, Wisdom, Charisma, and immunity to being
      charmed and frightened. It otherwise uses the possessed target's
      statistics, but doesn't gain access to the target's knowledge, class
      features, or proficiencies.
      
      The possession lasts until the body drops to 0 hit points, the ghost
      ends it as a bonus action, or the ghost is turned or forced out by an
      effect like the dispel evil and good spell. When the possession ends,
      the ghost reappears in an unoccupied space within 5 ft. of the body.
      The target is immune to this ghost's Possession for 24 hours after
      succeeding on the saving throw or after the possession ends.
    """
    name = "Ghost"
    description = "Medium undead, any alignment"
    challenge_rating = 4
    armor_class = 11
    skills = ""
    senses = "Darkvision 60 ft., Passive Perception 11"
    languages = "any languages it knew in life"
    strength = Ability(7)
    dexterity = Ability(13)
    constitution = Ability(10)
    intelligence = Ability(10)
    wisdom = Ability(12)
    charisma = Ability(17)
    speed = 0
    swim_speed = 0
    fly_speed = 40
    climb_speed = 0
    hp_max = 45
    hit_dice = "10d8"
    spells = []


class Ghoul(Monster):
    """
    Bite.
      Melee Weapon Attack: +2 to hit, reach 5 ft., one creature. Hit: 9 (2d6
      + 2) piercing damage.
    Claws.
      Melee Weapon Attack: +4 to hit, reach 5 ft., one target. Hit: 7 (2d4 +
      2) slashing damage. If the target is a creature other than an elf or
      undead, it must succeed on a DC 10 Constitution saving throw or be
      paralyzed for 1 minute. The target can repeat the saving throw at the
      end of each of its turns, ending the effect on itself on a success.
    """
    name = "Ghoul"
    description = "Medium undead, chaotic evil"
    challenge_rating = 1
    armor_class = 12
    skills = ""
    senses = "Darkvision 60 ft., Passive Perception 10"
    languages = "Common"
    strength = Ability(13)
    dexterity = Ability(15)
    constitution = Ability(10)
    intelligence = Ability(7)
    wisdom = Ability(10)
    charisma = Ability(6)
    speed = 30
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 22
    hit_dice = "5d8"
    spells = []


class GiantApe(Monster):
    """
    Multiattack.
      The ape makes two fist attacks.
    Fist.
      Melee Weapon Attack: +9 to hit, reach 10 ft., one target. Hit: 22
      (3d10 + 6) bludgeoning damage.
    Rock.
      Ranged Weapon Attack: +9 to hit, range 50/100 ft., one target. Hit: 30
      (7d6 + 6) bludgeoning damage.
    """
    name = "Giant Ape"
    description = "Huge beast, unaligned"
    challenge_rating = 7
    armor_class = 12
    skills = "Athletics +9, Perception +4"
    senses = "Passive Perception 14"
    languages = ""
    strength = Ability(23)
    dexterity = Ability(14)
    constitution = Ability(18)
    intelligence = Ability(7)
    wisdom = Ability(12)
    charisma = Ability(7)
    speed = 40
    swim_speed = 0
    fly_speed = 0
    climb_speed = 40
    hp_max = 157
    hit_dice = "15d12"
    spells = []


class GiantBadger(Monster):
    """
    Keen Smell.
      The badger has advantage on Wisdom (Perception) checks that rely on
      smell.
    Multiattack.
      The badger makes two attacks: one with its bite and one with its
      claws.
    Bite.
      Melee Weapon Attack: +3 to hit, reach 5 ft., one target. Hit: 4 (1d6 +
      1) piercing damage.
    Claws.
      Melee Weapon Attack: +3 to hit, reach 5 ft., one target. Hit: 6 (2d4 +
      1) slashing damage.
    """
    name = "Giant Badger"
    description = "Medium beast, unaligned"
    challenge_rating = 0.25
    armor_class = 10
    skills = ""
    senses = "Darkvision 30 ft., Passive Perception 11"
    languages = ""
    strength = Ability(13)
    dexterity = Ability(10)
    constitution = Ability(15)
    intelligence = Ability(2)
    wisdom = Ability(12)
    charisma = Ability(5)
    speed = 30
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 13
    hit_dice = "2d8"
    spells = []


class GiantBat(Monster):
    """
    Echolocation.
      The bat can't use its blindsight while deafened.
    Keen Hearing.
      The bat has advantage on Wisdom (Perception) checks that rely on
      hearing.
    Bite.
      Melee Weapon Attack: +4 to hit, reach 5 ft., one creature. Hit: 5 (1d6
      + 2) piercing damage.
    """
    name = "Giant Bat"
    description = "Large beast, unaligned"
    challenge_rating = 0.25
    armor_class = 13
    skills = ""
    senses = "Blindsight 60 ft., Passive Perception 11"
    languages = ""
    strength = Ability(15)
    dexterity = Ability(16)
    constitution = Ability(11)
    intelligence = Ability(2)
    wisdom = Ability(12)
    charisma = Ability(6)
    speed = 10
    swim_speed = 0
    fly_speed = 60
    climb_speed = 0
    hp_max = 22
    hit_dice = "4d10"
    spells = []


class GiantBoar(Monster):
    """
    Charge.
      If the boar moves at least 20 ft. straight toward a target and then
      hits it with a tusk attack on the same turn, the target takes an extra
      7 (2d6) slashing damage. If the target is a creature, it must succeed
      on a DC 13 Strength saving throw or be knocked prone.
    Relentless.
      If the boar takes 10 damage or less that would reduce it to 0 hit
      points, it is reduced to 1 hit point instead.
    Tusk.
      Melee Weapon Attack: +5 to hit, reach 5 ft., one target. Hit: 10 (2d6
      + 3) slashing damage.
    """
    name = "Giant Boar"
    description = "Large beast, unaligned"
    challenge_rating = 2
    armor_class = 12
    skills = ""
    senses = "Passive Perception 8"
    languages = ""
    strength = Ability(17)
    dexterity = Ability(10)
    constitution = Ability(16)
    intelligence = Ability(2)
    wisdom = Ability(7)
    charisma = Ability(5)
    speed = 40
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 42
    hit_dice = "5d10"
    spells = []


class GiantCentipede(Monster):
    """
    Bite.
      Bite. Melee Weapon Attack: +4 to hit, reach 5 ft., one creature. Hit:
      4 (1d4 + 2) piercing damage, and the target must succeed on a DC 11
      Constitution saving throw or take 10 (3d6) poison damage. If the
      poison damage reduces the target to 0 hit points, the target is stable
      but poisoned for 1 hour, even after regaining hit points, and is
      paralyzed while poisoned in this way.
    """
    name = "Giant Centipede"
    description = "Small beast, unaligned"
    challenge_rating = 0.25
    armor_class = 13
    skills = ""
    senses = "Blindsight 30 ft., Passive Perception 8"
    languages = ""
    strength = Ability(5)
    dexterity = Ability(14)
    constitution = Ability(12)
    intelligence = Ability(1)
    wisdom = Ability(7)
    charisma = Ability(3)
    speed = 30
    swim_speed = 0
    fly_speed = 0
    climb_speed = 30
    hp_max = 4
    hit_dice = "1d6"
    spells = []


class GiantConstrictorSnake(Monster):
    """
    Bite.
      Melee Weapon Attack: +6 to hit, reach 10 ft., one creature. Hit: 11
      (2d6 + 4) piercing damage.
    Constrict.
      Melee Weapon Attack: +6 to hit, reach 5 ft., one creature. Hit: 13
      (2d8 + 4) bludgeoning damage, and the target is grappled (escape DC
      16). Until this grapple ends, the creature is restrained, and the
      snake can't constrict another target.
    """
    name = "Giant Constrictor Snake"
    description = "Huge beast, unaligned"
    challenge_rating = 2
    armor_class = 12
    skills = "Perception +2"
    senses = "Blindsight 10 ft., Passive Perception 12"
    languages = ""
    strength = Ability(19)
    dexterity = Ability(14)
    constitution = Ability(12)
    intelligence = Ability(1)
    wisdom = Ability(10)
    charisma = Ability(3)
    speed = 30
    swim_speed = 30
    fly_speed = 0
    climb_speed = 0
    hp_max = 60
    hit_dice = "8d12"
    spells = []


class GiantCrab(Monster):
    """
    Amphibious.
      The crab can breathe air and water.
    Claw.
      Melee Weapon Attack: +3 to hit, reach 5 ft., one target. Hit: 4 (1d6 +
      1) bludgeoning damage, and the target is grappled (escape DC 11). The
      crab has two claws, each of which can grapple only one target.
    """
    name = "Giant Crab"
    description = "Medium beast, unaligned"
    challenge_rating = 0.125
    armor_class = 15
    skills = "Stealth +4"
    senses = "Blindsight 30 ft., Passive Perception 9"
    languages = ""
    strength = Ability(13)
    dexterity = Ability(15)
    constitution = Ability(11)
    intelligence = Ability(1)
    wisdom = Ability(9)
    charisma = Ability(3)
    speed = 30
    swim_speed = 30
    fly_speed = 0
    climb_speed = 0
    hp_max = 13
    hit_dice = "3d8"
    spells = []


class GiantCrocodile(Monster):
    """
    Hold Breath.
      The crocodile can hold its breath for 30 minutes.
    Multiattack.
      The crocodile makes two attacks: one with its bite and one with its
      tail.
    Bite.
      Melee Weapon Attack: +8 to hit, reach 5 ft., one target. Hit: 21 (3d10
      + 5) piercing damage, and the target is grappled (escape DC 16). Until
      this grapple ends, the target is restrained, and the crocodile can't
      bite another target.
    Tail.
      Melee Weapon Attack: +8 to hit, reach 10 ft., one target not grappled
      by the crocodile. Hit: 14 (2d8 + 5) bludgeoning damage. If the target
      is a creature, it must succeed on a DC 16 Strength saving throw or be
      knocked prone.
    """
    name = "Giant Crocodile"
    description = "Huge beast, unaligned"
    challenge_rating = 5
    armor_class = 14
    skills = "Stealth +5"
    senses = "Passive Perception 10"
    languages = ""
    strength = Ability(21)
    dexterity = Ability(9)
    constitution = Ability(17)
    intelligence = Ability(2)
    wisdom = Ability(10)
    charisma = Ability(7)
    speed = 30
    swim_speed = 50
    fly_speed = 0
    climb_speed = 0
    hp_max = 85
    hit_dice = "9d12"
    spells = []


class GiantEagle(Monster):
    """
    Keen Sight.
      The eagle has advantage on Wisdom (Perception) checks that rely on
      sight.
    Multiattack.
      The eagle makes two attacks: one with its beak and one with its
      talons.
    Beak.
      Melee Weapon Attack: +5 to hit, reach 5 ft., one target. Hit: 6 (1d6 +
      3) piercing damage.
    Talons.
      Melee Weapon Attack: +5 to hit, reach 5 ft., one target. Hit: 10 (2d6
      + 3) slashing damage.
    """
    name = "Giant Eagle"
    description = "Large beast, neutral good"
    challenge_rating = 1
    armor_class = 13
    skills = "Perception +4"
    senses = "Passive Perception 14"
    languages = "Giant Eagle, understands Common and Auran but can't speak"
    strength = Ability(16)
    dexterity = Ability(17)
    constitution = Ability(13)
    intelligence = Ability(8)
    wisdom = Ability(14)
    charisma = Ability(10)
    speed = 10
    swim_speed = 0
    fly_speed = 80
    climb_speed = 0
    hp_max = 26
    hit_dice = "4d10"
    spells = []


class GiantElk(Monster):
    """
    Charge.
      If the elk moves at least 20 ft. straight toward a target and then
      hits it with a ram attack on the same turn, the target takes an extra
      7 (2d6) damage. If the target is a creature, it must succeed on a DC
      14 Strength saving throw or be knocked prone.
    Ram.
      Melee Weapon Attack: +6 to hit, reach 10 ft., one target. Hit: 11 (2d6
      + 4) bludgeoning damage.
    Hooves.
      Melee Weapon Attack: +6 to hit, reach 5 ft., one prone creature. Hit:
      22 (4d8 + 4) bludgeoning damage.
    """
    name = "Giant Elk"
    description = "Huge beast, unaligned"
    challenge_rating = 2
    armor_class = 15
    skills = "Perception +4"
    senses = "Passive Perception 14"
    languages = "Giant Elk, understands Common, Elvish, and Sylvan but can't speak"
    strength = Ability(19)
    dexterity = Ability(16)
    constitution = Ability(14)
    intelligence = Ability(7)
    wisdom = Ability(14)
    charisma = Ability(10)
    speed = 60
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 42
    hit_dice = "5d12"
    spells = []


class GiantFireBeetle(Monster):
    """
    Illumination.
      The beetle sheds bright light in a 10-foot radius and dim light for an
      additional 10 ft..
    Bite.
      Melee Weapon Attack: +1 to hit, reach 5 ft., one target. Hit: 2 (1d6 -
      1) slashing damage.
    """
    name = "Giant Fire Beetle"
    description = "Small beast, unaligned"
    challenge_rating = 0
    armor_class = 13
    skills = ""
    senses = "Blindsight 30 ft., Passive Perception 8"
    languages = ""
    strength = Ability(8)
    dexterity = Ability(10)
    constitution = Ability(12)
    intelligence = Ability(1)
    wisdom = Ability(7)
    charisma = Ability(3)
    speed = 30
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 4
    hit_dice = "1d6"
    spells = []


class GiantFrog(Monster):
    """
    Amphibious.
      The frog can breathe air and water
    Standing Leap.
      The frog's long jump is up to 20 ft. and its high jump is up to 10
      ft., with or without a running start.
    Bite.
      Melee Weapon Attack: +3 to hit, reach 5 ft., one target. Hit: 4 (1d6 +
      1) piercing damage, and the target is grappled (escape DC 11). Until
      this grapple ends, the target is restrained, and the frog can't bite
      another target.
    Swallow.
      The frog makes one bite attack against a Small or smaller target it is
      grappling. If the attack hits, the target is swallowed, and the
      grapple ends. The swallowed target is blinded and restrained, it has
      total cover against attacks and other effects outside the frog, and it
      takes 5 (2d4) acid damage at the start of each of the frog's turns.
      The frog can have only one target swallowed at a time. If the frog
      dies, a swallowed creature is no longer restrained by it and can
      escape from the corpse using 5 ft. of movement, exiting prone.
    """
    name = "Giant Frog"
    description = "Medium beast, unaligned"
    challenge_rating = 0.25
    armor_class = 11
    skills = "Perception +2, Stealth +3"
    senses = "Darkvision 30 ft., Passive Perception 12"
    languages = ""
    strength = Ability(12)
    dexterity = Ability(13)
    constitution = Ability(11)
    intelligence = Ability(2)
    wisdom = Ability(10)
    charisma = Ability(3)
    speed = 30
    swim_speed = 30
    fly_speed = 0
    climb_speed = 0
    hp_max = 18
    hit_dice = "4d8"
    spells = []


class GiantGoat(Monster):
    """
    Charge.
      If the goat moves at least 20 ft. straight toward a target and then
      hits it with a ram attack on the same turn, the target takes an extra
      5 (2d4) bludgeoning damage. If the target is a creature, it must
      succeed on a DC 13 Strength saving throw or be knocked prone.
    Sure-Footed.
      The goat has advantage on Strength and Dexterity saving throws made
      against effects that would knock it prone.
    Ram.
      Melee Weapon Attack: +5 to hit, reach 5 ft., one target. Hit: 8 (2d4 +
      3) bludgeoning damage.
    """
    name = "Giant Goat"
    description = "Large beast, unaligned"
    challenge_rating = 0.5
    armor_class = 11
    skills = ""
    senses = "Passive Perception 11"
    languages = ""
    strength = Ability(17)
    dexterity = Ability(11)
    constitution = Ability(12)
    intelligence = Ability(3)
    wisdom = Ability(12)
    charisma = Ability(6)
    speed = 40
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 19
    hit_dice = "3d10"
    spells = []


class GiantHyena(Monster):
    """
    Rampage.
      When the hyena reduces a creature to 0 hit points with a melee attack
      on its turn, the hyena can take a bonus action to move up to half its
      speed and make a bite attack.
    Bite.
      Melee Weapon Attack: +5 to hit, reach 5 ft., one target. Hit: 10 (2d6
      + 3) piercing damage.
    """
    name = "Giant Hyena"
    description = "Large beast, unaligned"
    challenge_rating = 1
    armor_class = 12
    skills = "Perception +3"
    senses = "Passive Perception 13"
    languages = ""
    strength = Ability(16)
    dexterity = Ability(14)
    constitution = Ability(14)
    intelligence = Ability(2)
    wisdom = Ability(12)
    charisma = Ability(7)
    speed = 50
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 45
    hit_dice = "6d10"
    spells = []


class GiantLizard(Monster):
    """
    Variant: Hold Breath.
      The lizard can hold its breath for 15 minutes. (A lizard that has this
      trait also has a swimming speed of 30 feet.)
    Variant: Spider Climb.
      The lizard can climb difficult surfaces, including upside down on
      ceilings, without needing to make an ability check.
    Bite.
      Melee Weapon Attack: +4 to hit, reach 5 ft., one target. Hit: 6 (1d8 +
      2) piercing damage.
    """
    name = "Giant Lizard"
    description = "Large beast, unaligned"
    challenge_rating = 0.25
    armor_class = 12
    skills = ""
    senses = "Darkvision 30 ft., Passive Perception 10"
    languages = ""
    strength = Ability(15)
    dexterity = Ability(12)
    constitution = Ability(13)
    intelligence = Ability(2)
    wisdom = Ability(10)
    charisma = Ability(5)
    speed = 30
    swim_speed = 0
    fly_speed = 0
    climb_speed = 30
    hp_max = 19
    hit_dice = "3d10"
    spells = []


class GiantOctopus(Monster):
    """
    Hold Breath.
      While out of water, the octopus can hold its breath for 1 hour.
    Underwater Camouflage.
      The octopus has advantage on Dexterity (Stealth) checks made while
      underwater.
    Water Breathing.
      The octopus can breathe only underwater.
    Tentacles.
      Melee Weapon Attack: +5 to hit, reach 15 ft., one target. Hit: 10 (2d6
      + 3) bludgeoning damage. If the target is a creature, it is grappled
      (escape DC 16). Until this grapple ends, the target is restrained, and
      the octopus can't use its tentacles on another target.
    Ink Cloud.
      A 20-foot-radius cloud of ink extends all around the octopus if it is
      underwater. The area is heavily obscured for 1 minute, although a
      significant current can disperse the ink. After releasing the ink, the
      octopus can use the Dash action as a bonus action.
    """
    name = "Giant Octopus"
    description = "Large beast, unaligned"
    challenge_rating = 1
    armor_class = 11
    skills = "Perception +4, Stealth +5"
    senses = "Darkvision 60 ft., Passive Perception 14"
    languages = ""
    strength = Ability(17)
    dexterity = Ability(13)
    constitution = Ability(13)
    intelligence = Ability(4)
    wisdom = Ability(10)
    charisma = Ability(4)
    speed = 10
    swim_speed = 60
    fly_speed = 0
    climb_speed = 0
    hp_max = 52
    hit_dice = "8d10"
    spells = []


class GiantOwl(Monster):
    """
    Flyby.
      The owl doesn't provoke opportunity attacks when it flies out of an
      enemy's reach.
    Keen Hearing and Sight.
      The owl has advantage on Wisdom (Perception) checks that rely on
      hearing or sight.
    Talons.
      Melee Weapon Attack: +3 to hit, reach 5 ft., one target. Hit: 8 (2d6 +
      1) slashing damage.
    """
    name = "Giant Owl"
    description = "Large beast, neutral"
    challenge_rating = 0.25
    armor_class = 12
    skills = "Perception +5, Stealth +4"
    senses = "Darkvision 120 ft., Passive Perception 15"
    languages = "Giant Owl, understands Common, Elvish, and Sylvan but can't speak"
    strength = Ability(13)
    dexterity = Ability(15)
    constitution = Ability(12)
    intelligence = Ability(8)
    wisdom = Ability(13)
    charisma = Ability(10)
    speed = 5
    swim_speed = 0
    fly_speed = 60
    climb_speed = 0
    hp_max = 19
    hit_dice = "3d10"
    spells = []


class GiantPoisonousSnake(Monster):
    """
    Bite.
      Melee Weapon Attack: +6 to hit, reach 10 ft., one target. Hit: 6 (1d4
      + 4) piercing damage, and the target must make a DC 11 Constitution
      saving throw, taking 10 (3d6) poison damage on a failed save, or half
      as much damage on a successful one.
    """
    name = "Giant Poisonous Snake"
    description = "Medium beast, unaligned"
    challenge_rating = 0.25
    armor_class = 14
    skills = "Perception +2"
    senses = "Blindsight 10 ft., Passive Perception 12"
    languages = ""
    strength = Ability(10)
    dexterity = Ability(18)
    constitution = Ability(13)
    intelligence = Ability(2)
    wisdom = Ability(10)
    charisma = Ability(3)
    speed = 30
    swim_speed = 30
    fly_speed = 0
    climb_speed = 0
    hp_max = 11
    hit_dice = "2d8"
    spells = []


class GiantRat(Monster):
    """
    Keen Smell.
      The rat has advantage on Wisdom (Perception) checks that rely on
      smell.
    Pack Tactics.
      The rat has advantage on an attack roll against a creature if at least
      one of the rat's allies is within 5 ft. of the creature and the ally
      isn't incapacitated.
    Bite.
      Melee Weapon Attack: +4 to hit, reach 5 ft., one target. Hit: 4 (1d4 +
      2) piercing damage.
    """
    name = "Giant Rat"
    description = "Small beast, unaligned"
    challenge_rating = 0.125
    armor_class = 12
    skills = ""
    senses = "Darkvision 60 ft., Passive Perception 10"
    languages = ""
    strength = Ability(7)
    dexterity = Ability(15)
    constitution = Ability(11)
    intelligence = Ability(2)
    wisdom = Ability(10)
    charisma = Ability(4)
    speed = 30
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 7
    hit_dice = "2d6"
    spells = []


class GiantRatDiseased(Monster):
    """
    Bite.
      Melee Weapon Attack: +4 to hit, reach 5 ft., one target. Hit: 3 (1d4 +
      2) piercing damage. If the target is a creature, it must succeed on a
      DC 10 Constitution saving throw or contract a disease. Until the
      disease is cured, the target can't regain hit points except by magical
      means, and the target's hit point maximum decreases by 3 (1d6) every
      24 hours. If the target's hit point maximum drops to 0 as a result of
      this disease, the target dies.
    """
    name = "Giant Rat (Diseased)"
    description = "Small beast, unaligned"
    challenge_rating = 0.125
    armor_class = 12
    skills = ""
    senses = "Darkvision 60 ft., Passive Perception 10"
    languages = ""
    strength = Ability(7)
    dexterity = Ability(15)
    constitution = Ability(11)
    intelligence = Ability(2)
    wisdom = Ability(10)
    charisma = Ability(4)
    speed = 30
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 7
    hit_dice = "2d6"
    spells = []


class GiantScorpion(Monster):
    """
    Claw.
      Melee Weapon Attack: +4 to hit, reach 5 ft., one target. Hit: 6 (1d8 +
      2) bludgeoning damage, and the target is grappled (escape DC 12). The
      scorpion has two claws, each of which can grapple only one target.
    Multiattack.
      The scorpion makes three attacks: two with its claws and one with its
      sting.
    Sting.
      Melee Weapon Attack: +4 to hit, reach 5 ft., one creature. Hit: 7
      (1d10 + 2) piercing damage, and the target must make a DC 12
      Constitution saving throw, taking 22 (4d10) poison damage on a failed
      save, or half as much damage on a successful one.
    """
    name = "Giant Scorpion"
    description = "Large beast, unaligned"
    challenge_rating = 3
    armor_class = 15
    skills = ""
    senses = "Blindsight 60 ft., Passive Perception 9"
    languages = ""
    strength = Ability(15)
    dexterity = Ability(13)
    constitution = Ability(15)
    intelligence = Ability(1)
    wisdom = Ability(9)
    charisma = Ability(3)
    speed = 40
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 52
    hit_dice = "7d10"
    spells = []


class GiantSeaHorse(Monster):
    """
    Charge.
      If the sea horse moves at least 20 ft. straight toward a target and
      then hits it with a ram attack on the same turn, the target takes an
      extra 7 (2d6) bludgeoning damage. If the target is a creature, it must
      succeed on a DC 11 Strength saving throw or be knocked prone.
    Water Breathing.
      The sea horse can breathe only underwater.
    Ram.
      Melee Weapon Attack: +3 to hit, reach 5 ft., one target. Hit: 4 (1d6 +
      1) bludgeoning damage.
    """
    name = "Giant Sea Horse"
    description = "Large beast, unaligned"
    challenge_rating = 0.5
    armor_class = 13
    skills = ""
    senses = "Passive Perception 11"
    languages = ""
    strength = Ability(12)
    dexterity = Ability(15)
    constitution = Ability(11)
    intelligence = Ability(2)
    wisdom = Ability(12)
    charisma = Ability(5)
    speed = 0
    swim_speed = 40
    fly_speed = 0
    climb_speed = 0
    hp_max = 16
    hit_dice = "3d10"
    spells = []


class GiantShark(Monster):
    """
    Blood Frenzy.
      The shark has advantage on melee attack rolls against any creature
      that doesn't have all its hit points.
    Water Breathing.
      The shark can breathe only underwater.
    Bite.
      Weapon Attack: +9 to hit, reach 5 ft., one target. Hit: 22 (3d10 + 6)
      piercing damage.
    """
    name = "Giant Shark"
    description = "Huge beast, unaligned"
    challenge_rating = 5
    armor_class = 13
    skills = "Perception +3"
    senses = "Blindsight 60 ft., Passive Perception 13"
    languages = ""
    strength = Ability(23)
    dexterity = Ability(11)
    constitution = Ability(21)
    intelligence = Ability(1)
    wisdom = Ability(10)
    charisma = Ability(5)
    speed = 0
    swim_speed = 50
    fly_speed = 0
    climb_speed = 0
    hp_max = 126
    hit_dice = "11d12"
    spells = []


class GiantSpider(Monster):
    """
    Spider Climb.
      The spider can climb difficult surfaces, including upside down on
      ceilings, without needing to make an ability check.
    Web Sense.
      While in contact with a web, the spider knows the exact location of
      any other creature in contact with the same web.
    Web Walker.
      The spider ignores movement restrictions caused by webbing.
    Bite.
      Melee Weapon Attack: +5 to hit, reach 5 ft., one creature. Hit: 7 (1d8
      + 3) piercing damage, and the target must make a DC 11 Constitution
      saving throw, taking 9 (2d8) poison damage on a failed save, or half
      as much damage on a successful one. If the poison damage reduces the
      target to 0 hit points, the target is stable but poisoned for 1 hour,
      even after regaining hit points, and is paralyzed while poisoned in
      this way.
    Web.
      Ranged Weapon Attack: +5 to hit, range 30/60 ft., one creature. Hit:
      The target is restrained by webbing. As an action, the restrained
      target can make a DC 12 Strength check, bursting the webbing on a
      success. The webbing can also be attacked and destroyed (AC 10; hp 5;
      vulnerability to fire damage; immunity to bludgeoning, poison, and
      psychic damage).
    """
    name = "Giant Spider"
    description = "Large beast, unaligned"
    challenge_rating = 1
    armor_class = 14
    skills = "Stealth +7"
    senses = "Blindsight 10 ft., Darkvision 60 ft., Passive Perception 10"
    languages = ""
    strength = Ability(14)
    dexterity = Ability(16)
    constitution = Ability(12)
    intelligence = Ability(2)
    wisdom = Ability(11)
    charisma = Ability(4)
    speed = 30
    swim_speed = 0
    fly_speed = 0
    climb_speed = 30
    hp_max = 26
    hit_dice = "4d10"
    spells = []


class GiantToad(Monster):
    """
    Amphibious.
      The toad can breathe air and water
    Standing Leap.
      The toad's long jump is up to 20 ft. and its high jump is up to 10
      ft., with or without a running start.
    Bite.
      Melee Weapon Attack: +4 to hit, reach 5 ft., one target. Hit: 7 (1d10
      + 2) piercing damage plus 5 (1d10) poison damage, and the target is
      grappled (escape DC 13). Until this grapple ends, the target is
      restrained, and the toad can't bite another target.
    Swallow.
      The toad makes one bite attack against a Medium or smaller target it
      is grappling. If the attack hits, the target is swallowed, and the
      grapple ends. The swallowed target is blinded and restrained, it has
      total cover against attacks and other effects outside the toad, and it
      takes 10 (3d6) acid damage at the start of each of the toad's turns.
      The toad can have only one target swallowed at a time.
      
      If the toad dies, a swallowed creature is no longer restrained by it
      and can escape from the corpse using 5 feet of movement, exiting
      prone.
    """
    name = "Giant Toad"
    description = "Large beast, unaligned"
    challenge_rating = 1
    armor_class = 11
    skills = ""
    senses = "Darkvision 30 ft., Passive Perception 10"
    languages = ""
    strength = Ability(15)
    dexterity = Ability(13)
    constitution = Ability(13)
    intelligence = Ability(2)
    wisdom = Ability(10)
    charisma = Ability(3)
    speed = 20
    swim_speed = 40
    fly_speed = 0
    climb_speed = 0
    hp_max = 39
    hit_dice = "6d10"
    spells = []


class GiantVulture(Monster):
    """
    Keen Sight and Smell.
      The vulture has advantage on Wisdom (Perception) checks that rely on
      sight or smell.
    Pack Tactics.
      The vulture has advantage on an attack roll against a creature if at
      least one of the vulture's allies is within 5 ft. of the creature and
      the ally isn't incapacitated.
    Multiattack.
      The vulture makes two attacks: one with its beak and one with its
      talons.
    Beak.
      Melee Weapon Attack: +4 to hit, reach 5 ft., one target. Hit: 7 (2d4 +
      2) piercing damage.
    Talons.
      Melee Weapon Attack: +4 to hit, reach 5 ft., one target. Hit: 9 (2d6 +
      2) slashing damage.
    """
    name = "Giant Vulture"
    description = "Large beast, neutral evil"
    challenge_rating = 1
    armor_class = 10
    skills = "Perception +3"
    senses = "Passive Perception 13"
    languages = "understands Common but can't speak"
    strength = Ability(15)
    dexterity = Ability(10)
    constitution = Ability(15)
    intelligence = Ability(6)
    wisdom = Ability(12)
    charisma = Ability(7)
    speed = 10
    swim_speed = 0
    fly_speed = 60
    climb_speed = 0
    hp_max = 22
    hit_dice = "3d10"
    spells = []


class GiantWasp(Monster):
    """
    Sting.
      Sting. Melee Weapon Attack: +4 to hit, reach 5 ft., one creature. Hit:
      5 (1d6 + 2) piercing damage, and the target must make a DC 11
      Constitution saving throw, taking 10 (3d6) poison damage on a failed
      save, or half as much damage on a successful one. If the poison damage
      reduces the target to 0 hit points, the target is stable but poisoned
      for 1 hour, even after regaining hit points, and is paralyzed while
      poisoned in this way.
    """
    name = "Giant Wasp"
    description = "Medium beast, unaligned"
    challenge_rating = 0.5
    armor_class = 12
    skills = ""
    senses = "Passive Perception 10"
    languages = ""
    strength = Ability(10)
    dexterity = Ability(14)
    constitution = Ability(10)
    intelligence = Ability(1)
    wisdom = Ability(10)
    charisma = Ability(3)
    speed = 10
    swim_speed = 50
    fly_speed = 50
    climb_speed = 0
    hp_max = 13
    hit_dice = "3d8"
    spells = []


class GiantWeasel(Monster):
    """
    Keen Hearing and Smell.
      The weasel has advantage on Wisdom (Perception) checks that rely on
      hearing or smell.
    Bite.
      Weapon Attack: +5 to hit, reach 5 ft., one target. Hit: 5 (1d4 + 3)
      piercing damage.
    """
    name = "Giant Weasel"
    description = "Medium beast, unaligned"
    challenge_rating = 0.125
    armor_class = 13
    skills = "Perception +3, Stealth +5"
    senses = "Darkvision 60 ft., Passive Perception 13"
    languages = ""
    strength = Ability(11)
    dexterity = Ability(16)
    constitution = Ability(10)
    intelligence = Ability(4)
    wisdom = Ability(12)
    charisma = Ability(5)
    speed = 40
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 9
    hit_dice = "2d8"
    spells = []


class GiantWolfSpider(Monster):
    """
    Spider Climb.
      The spider can climb difficult surfaces, including upside down on
      ceilings, without needing to make an ability check.
    Web Sense.
      While in contact with a web, the spider knows the exact location of
      any other creature in contact with the same web.
    Web Walker.
      The spider ignores movement restrictions caused by webbing.
    Bite.
      Weapon Attack: +3 to hit, reach 5 ft., one creature. Hit: 4 (1d6 + 1)
      piercing damage, and the target must make a DC 11 Constitution saving
      throw, taking 7 (2d6) poison damage on a failed save, or half as much
      damage on a successful one. If the poison damage reduces the target to
      0 hit points, the target is stable but poisoned for 1 hour, even after
      regaining hit points, and is paralyzed while poisoned in this way.
    """
    name = "Giant Wolf Spider"
    description = "Medium beast, unaligned"
    challenge_rating = 0.25
    armor_class = 13
    skills = "Perception +3, Stealth +7"
    senses = "Blindsight 10 ft., Darkvision 60 ft., Passive Perception 13"
    languages = ""
    strength = Ability(12)
    dexterity = Ability(16)
    constitution = Ability(13)
    intelligence = Ability(3)
    wisdom = Ability(12)
    charisma = Ability(4)
    speed = 40
    swim_speed = 0
    fly_speed = 0
    climb_speed = 40
    hp_max = 11
    hit_dice = "2d8"
    spells = []


class GibberingMouther(Monster):
    """
    Aberrant Ground.
      The ground in a 10-foot radius around the mouther is doughlike
      difficult terrain. Each creature that starts its turn in that area
      must succeed on a DC 10 Strength saving throw or have its speed
      reduced to 0 until the start of its next turn.
    Gibbering.
      The mouther babbles incoherently while it can see any creature and
      isn't incapacitated. Each creature that starts its turn within 20 feet
      of the mouther and can hear the gibbering must succeed on a DC 10
      Wisdom saving throw. On a failure, the creature can't take reactions
      until the start of its next turn and rolls a d8 to determine what it
      does during its turn. On a 1 to 4, the creature does nothing. On a 5
      or 6, the creature takes no action or bonus action and uses all its
      movement to move in a randomly determined direction. On a 7 or 8, the
      creature makes a melee attack against a randomly determined creature
      within its reach or does nothing if it can't make such an attack.
    Multiattack.
      The gibbering mouther makes one bite attack and, if it can, uses its
      Blinding Spittle.
    Bites.
      Melee Weapon Attack: +2 to hit, reach 5 ft., one creature. Hit: 17
      (5d6) piercing damage. If the target is Medium or smaller, it must
      succeed on a DC 10 Strength saving throw or be knocked prone. If the
      target is killed by this damage, it is absorbed into the mouther.
    Blinding Spittle.
      The mouther spits a chemical glob at a point it can see within 15 feet
      of it. The glob explodes in a blinding flash of light on impact. Each
      creature within 5 feet of the flash must succeed on a DC 13 Dexterity
      saving throw or be blinded until the end of the mouther's next turn.
    """
    name = "Gibbering Mouther"
    description = "Medium aberration, neutral"
    challenge_rating = 2
    armor_class = 9
    skills = ""
    senses = "Darkvision 60 ft., Passive Perception 10"
    languages = ""
    strength = Ability(10)
    dexterity = Ability(8)
    constitution = Ability(16)
    intelligence = Ability(3)
    wisdom = Ability(10)
    charisma = Ability(6)
    speed = 10
    swim_speed = 10
    fly_speed = 0
    climb_speed = 0
    hp_max = 67
    hit_dice = "9d8"
    spells = []


class Glabrezu(Monster):
    """
    Innate Spellcasting.
      The glabrezu's spellcasting ability is Intelligence (spell save DC
      16). The glabrezu can innately cast the following spells, requiring no
      material components:
      
      At will: darkness, detect magic, dispel magic
      
      1/day each: confusion, fly, power word stun
    Magic Resistance.
      The glabrezu has advantage on saving throws against spells and other
      magical effects.
    Multiattack.
      The glabrezu makes four attacks: two with its pincers and two with its
      fists. Alternatively, it makes two attacks with its pincers and casts
      one spell.
    Pincer.
      Melee Weapon Attack: +9 to hit, reach 10 ft., one target. Hit: 16
      (2d10 + 5) bludgeoning damage. If the target is a Medium or smaller
      creature, it is grappled (escape DC 15). The glabrezu has two pincers,
      each of which can grapple only one target.
    Fist.
      Melee Weapon Attack: +9 to hit, reach 5 ft., one target. Hit: 7 (2d4 +
      2) bludgeoning damage.
    Variant: Summon Demon.
      The demon chooses what to summon and attempts a magical summoning.
      
      A glabrezu has a 30 percent chance of summoning 1d3 vrocks, 1d2
      hezrous, or one glabrezu.
      
      A summoned demon appears in an unoccupied space within 60 feet of its
      summoner, acts as an ally of its summoner, and can't summon other
      demons. It remains for 1 minute, until it or its summoner dies, or
      until its summoner dismisses it as an action.
    """
    name = "Glabrezu"
    description = "Large fiend, chaotic evil"
    challenge_rating = 9
    armor_class = 17
    skills = ""
    senses = "Truesight 120 ft., Passive Perception 13"
    languages = "Abyssal, telepathy 120 ft."
    strength = Ability(20)
    dexterity = Ability(15)
    constitution = Ability(21)
    intelligence = Ability(19)
    wisdom = Ability(17)
    charisma = Ability(16)
    speed = 40
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 157
    hit_dice = "15d10"
    spells = []


class Gladiator(Monster):
    """
    Brave.
      The gladiator has advantage on saving throws against being frightened.
    Brute.
      A melee weapon deals one extra die of its damage when the gladiator
      hits with it (included in the attack).
    Multiattack.
      The gladiator makes three melee attacks or two ranged attacks.
    Spear.
      Melee or Ranged Weapon Attack: +7 to hit, reach 5 ft. and range 20/60
      ft., one target. Hit: 11 (2d6 + 4) piercing damage, or 13 (2d8 + 4)
      piercing damage if used with two hands to make a melee attack.
    Shield Bash.
      Melee Weapon Attack: +7 to hit, reach 5 ft., one creature. Hit: 9 (2d4
      + 4) bludgeoning damage. If the target is a Medium or smaller
      creature, it must succeed on a DC 15 Strength saving throw or be
      knocked prone.
    """
    name = "Gladiator"
    description = "Medium humanoid, any alignment"
    challenge_rating = 5
    armor_class = 16
    skills = "Athletics +10, Intimidation +5"
    senses = "Passive Perception 11"
    languages = "any one language (usually Common)"
    strength = Ability(18)
    dexterity = Ability(15)
    constitution = Ability(16)
    intelligence = Ability(10)
    wisdom = Ability(12)
    charisma = Ability(15)
    speed = 30
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 112
    hit_dice = "15d8"
    spells = []


class Gnoll(Monster):
    """
    Rampage.
      When the gnoll reduces a creature to 0 hit points with a melee attack
      on its turn, the gnoll can take a bonus action to move up to half its
      speed and make a bite attack.
    Bite.
      Melee Weapon Attack: +4 to hit, reach 5 ft., one creature. Hit: 4 (1d4
      + 2) piercing damage.
    Spear.
      Melee or Ranged Weapon Attack: +4 to hit, reach 5 ft. or range 20/60
      ft., one target. Hit: 5 (1d6 + 2) piercing damage, or 6 (1d8 + 2)
      piercing damage if used with two hands to make a melee attack.
    Longbow.
      Ranged Weapon Attack: +3 to hit, range 150/600 ft., one target. Hit: 5
      (1d8 + 1) piercing damage.
    """
    name = "Gnoll"
    description = "Medium humanoid, chaotic evil"
    challenge_rating = 0.5
    armor_class = 15
    skills = ""
    senses = "Darkvision 60 ft., Passive Perception 10"
    languages = "Gnoll"
    strength = Ability(14)
    dexterity = Ability(12)
    constitution = Ability(11)
    intelligence = Ability(6)
    wisdom = Ability(10)
    charisma = Ability(7)
    speed = 30
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 22
    hit_dice = "5d8"
    spells = []


class Goat(Monster):
    """
    Charge.
      If the goat moves at least 20 ft. straight toward a target and then
      hits it with a ram attack on the same turn, the target takes an extra
      2 (1d4) bludgeoning damage. If the target is a creature, it must
      succeed on a DC 10 Strength saving throw or be knocked prone.
    Sure-Footed.
      The goat has advantage on Strength and Dexterity saving throws made
      against effects that would knock it prone.
    Ram.
      Melee Weapon Attack: +3 to hit, reach 5 ft., one target. Hit: 3 (1d4 +
      1) bludgeoning damage.
    """
    name = "Goat"
    description = "Medium beast, unaligned"
    challenge_rating = 0
    armor_class = 10
    skills = ""
    senses = "Passive Perception 10"
    languages = ""
    strength = Ability(12)
    dexterity = Ability(10)
    constitution = Ability(11)
    intelligence = Ability(2)
    wisdom = Ability(10)
    charisma = Ability(5)
    speed = 40
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 4
    hit_dice = "1d8"
    spells = []


class Goblin(Monster):
    """
    Nimble Escape.
      The goblin can take the Disengage or Hide action as a bonus action on
      each of its turns.
    Scimitar.
      Melee Weapon Attack: +4 to hit, reach 5 ft., one target. Hit: 5 (1d6 +
      2) slashing damage.
    Shortbow.
      Ranged Weapon Attack: +4 to hit, range 80/320 ft., one target. Hit: 5
      (1d6 + 2) piercing damage.
    """
    name = "Goblin"
    description = "Small humanoid, neutral evil"
    challenge_rating = 0.25
    armor_class = 15
    skills = "Stealth +6"
    senses = "Darkvision 60 ft., Passive Perception 9"
    languages = "Common, Goblin"
    strength = Ability(8)
    dexterity = Ability(14)
    constitution = Ability(10)
    intelligence = Ability(10)
    wisdom = Ability(8)
    charisma = Ability(8)
    speed = 30
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 7
    hit_dice = "2d6"
    spells = []


class GoldDragonWyrmling(Monster):
    """
    Amphibious.
      The dragon can breathe air and water.
    Bite.
      Melee Weapon Attack: +6 to hit, reach 5 ft., one target. Hit: 9 (1d10
      + 4) piercing damage.
    Breath Weapons.
      The dragon uses one of the following breath weapons.
      
      Fire Breath. The dragon exhales fire in a 15-foot cone. Each creature
      in that area must make a DC 13 Dexterity saving throw, taking 22
      (4d10) fire damage on a failed save, or half as much damage on a
      successful one.
      
      Weakening Breath. The dragon exhales gas in a 15-foot cone. Each
      creature in that area must succeed on a DC 13 Strength saving throw or
      have disadvantage on Strength-based attack rolls, Strength checks, and
      Strength saving throws for 1 minute. A creature can repeat the saving
      throw at the end of each of its turns, ending the effect on itself on
      a success.
    """
    name = "Gold Dragon Wyrmling"
    description = "Medium dragon, lawful good"
    challenge_rating = 3
    armor_class = 17
    skills = "Perception +4, Stealth +4"
    senses = "Blindsight 10 ft., Darkvision 60 ft., Passive Perception 14"
    languages = "Draconic"
    strength = Ability(19)
    dexterity = Ability(14)
    constitution = Ability(17)
    intelligence = Ability(14)
    wisdom = Ability(11)
    charisma = Ability(16)
    speed = 30
    swim_speed = 30
    fly_speed = 60
    climb_speed = 0
    hp_max = 60
    hit_dice = "8d8"
    spells = []


class Gorgon(Monster):
    """
    Trampling Charge.
      If the gorgon moves at least 20 feet straight toward a creature and
      then hits it with a gore attack on the same turn, that target must
      succeed on a DC 16 Strength saving throw or be knocked prone. If the
      target is prone, the gorgon can make one attack with its hooves
      against it as a bonus action.
    Gore.
      Melee Weapon Attack: +8 to hit, reach 5 ft., one target. Hit: 18 (2d12
      + 5) piercing damage.
    Hooves.
      Melee Weapon Attack: +8 to hit, reach 5 ft., one target. Hit: 16 (2d10
      + 5) bludgeoning damage.
    Petrifying Breath.
      The gorgon exhales petrifying gas in a 30-foot cone. Each creature in
      that area must succeed on a DC 13 Constitution saving throw. On a
      failed save, a target begins to turn to stone and is restrained. The
      restrained target must repeat the saving throw at the end of its next
      turn. On a success, the effect ends on the target. On a failure, the
      target is petrified until freed by the greater restoration spell or
      other magic.
    """
    name = "Gorgon"
    description = "Large monstrosity, unaligned"
    challenge_rating = 5
    armor_class = 19
    skills = "Perception +4"
    senses = "Darkvision 60 ft., Passive Perception 14"
    languages = ""
    strength = Ability(20)
    dexterity = Ability(11)
    constitution = Ability(18)
    intelligence = Ability(2)
    wisdom = Ability(12)
    charisma = Ability(7)
    speed = 40
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 114
    hit_dice = "12d10"
    spells = []


class GrayOoze(Monster):
    """
    Amorphous.
      The ooze can move through a space as narrow as 1 inch wide without
      squeezing.
    Corrode Metal.
      Any nonmagical weapon made of metal that hits the ooze corrodes. After
      dealing damage, the weapon takes a permanent and cumulative -1 penalty
      to damage rolls. If its penalty drops to -5, the weapon is destroyed.
      Nonmagical ammunition made of metal that hits the ooze is destroyed
      after dealing damage.
      
      The ooze can eat through 2-inch-thick, nonmagical metal in 1 round.
    False Appearance.
      While the ooze remains motionless, it is indistinguishable from an
      oily pool or wet rock.
    Pseudopod.
      Melee Weapon Attack: +3 to hit, reach 5 ft., one target. Hit: 4 (1d6 +
      1) bludgeoning damage plus 7 (2d6) acid damage, and if the target is
      wearing nonmagical metal armor, its armor is partly corroded and takes
      a permanent and cumulative -1 penalty to the AC it offers. The armor
      is destroyed if the penalty reduces its AC to 10.
    """
    name = "Gray Ooze"
    description = "Medium ooze, unaligned"
    challenge_rating = 0.5
    armor_class = 8
    skills = "Stealth +2"
    senses = "Blindsight 60 ft. (blind beyond this radius), Passive Perception 8"
    languages = ""
    strength = Ability(12)
    dexterity = Ability(6)
    constitution = Ability(16)
    intelligence = Ability(1)
    wisdom = Ability(6)
    charisma = Ability(2)
    speed = 10
    swim_speed = 0
    fly_speed = 0
    climb_speed = 10
    hp_max = 22
    hit_dice = "3d8"
    spells = []


class GreenDragonWyrmling(Monster):
    """
    Amphibious.
      The dragon can breathe air and water.
    Bite.
      Melee Weapon Attack: +4 to hit, reach 5 ft., one target. Hit: 7 (1d10
      + 2) piercing damage plus 3 (1d6) poison damage.
    Poison Breath.
      The dragon exhales poisonous gas in a 15-foot cone. Each creature in
      that area must make a DC 11 Constitution saving throw, taking 21 (6d6)
      poison damage on a failed save, or half as much damage on a successful
      one.
    """
    name = "Green Dragon Wyrmling"
    description = "Medium dragon, lawful evil"
    challenge_rating = 2
    armor_class = 17
    skills = "Perception +4, Stealth +3"
    senses = "Blindsight 10 ft., Darkvision 60 ft., Passive Perception 14"
    languages = "Draconic"
    strength = Ability(15)
    dexterity = Ability(12)
    constitution = Ability(13)
    intelligence = Ability(14)
    wisdom = Ability(11)
    charisma = Ability(13)
    speed = 30
    swim_speed = 30
    fly_speed = 60
    climb_speed = 0
    hp_max = 38
    hit_dice = "7d8"
    spells = []


class GreenHag(Monster):
    """
    Amphibious.
      The hag can breathe air and water.
    Innate Spellcasting.
      The hag's innate spellcasting ability is Charisma (spell save DC 12).
      She can innately cast the following spells, requiring no material
      components:
      
  
      
      At will: dancing lights, minor illusion, vicious mockery
    Mimicry.
      The hag can mimic animal sounds and humanoid voices. A creature that
      hears the sounds can tell they are imitations with a successful DC 14
      Wisdom (Insight) check.
    Claws.
      Melee Weapon Attack: +6 to hit, reach 5 ft., one target. Hit: 13 (2d8
      + 4) slashing damage.
    Illusory Appearance.
      The hag covers herself and anything she is wearing or carrying with a
      magical illusion that makes her look like another creature of her
      general size and humanoid shape. The illusion ends if the hag takes a
      bonus action to end it or if she dies.
      
      The changes wrought by this effect fail to hold up to physical
      inspection. For example, the hag could appear to have smooth skin, but
      someone touching her would feel her rough flesh. Otherwise, a creature
      must take an action to visually inspect the illusion and succeed on a
      DC 20 Intelligence (Investigation) check to discern that the hag is
      disguised.
    Invisible Passage.
      The hag magically turns invisible until she attacks or casts a spell,
      or until her concentration ends (as if concentrating on a spell).
      While invisible, she leaves no physical evidence of her passage, so
      she can be tracked only by magic. Any equipment she wears or carries
      is invisible with her.
    """
    name = "Green Hag"
    description = "Medium fey, neutral evil"
    challenge_rating = 3
    armor_class = 17
    skills = "Arcana +3, Deception +4, Perception +4, Stealth +3"
    senses = "Darkvision 60 ft., Passive Perception 14"
    languages = "Common, Draconic, Sylvan"
    strength = Ability(18)
    dexterity = Ability(12)
    constitution = Ability(16)
    intelligence = Ability(13)
    wisdom = Ability(14)
    charisma = Ability(14)
    speed = 30
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 82
    hit_dice = "11d8"
    spells = []


class Grick(Monster):
    """
    Stone Camouflage.
      The grick has advantage on Dexterity (Stealth) checks made to hide in
      rocky terrain.
    Multiattack.
      The grick makes one attack with its tentacles. If that attack hits,
      the grick can make one beak attack against the same target.
    Tentacles.
      Melee Weapon Attack: +4 to hit, reach 5 ft., one target. Hit: 9 (2d6 +
      2) slashing damage.
    Beak.
      Melee Weapon Attack: +4 to hit, reach 5 ft., one target. Hit: 5 (1d6 +
      2) piercing damage.
    """
    name = "Grick"
    description = "Medium monstrosity, neutral"
    challenge_rating = 2
    armor_class = 14
    skills = ""
    senses = "Darkvision 60 ft., Passive Perception 12"
    languages = ""
    strength = Ability(14)
    dexterity = Ability(14)
    constitution = Ability(11)
    intelligence = Ability(3)
    wisdom = Ability(14)
    charisma = Ability(5)
    speed = 30
    swim_speed = 0
    fly_speed = 0
    climb_speed = 30
    hp_max = 27
    hit_dice = "6d8"
    spells = []


class Griffon(Monster):
    """
    Keen Sight.
      The griffon has advantage on Wisdom (Perception) checks that rely on
      sight.
    Multiattack.
      The griffon makes two attacks: one with its beak and one with its
      claws.
    Beak.
      Melee Weapon Attack: +6 to hit, reach 5 ft., one target. Hit: 8 (1d8 +
      4) piercing damage.
    Claws.
      Melee Weapon Attack: +6 to hit, reach 5 ft., one target. Hit: 11 (2d6
      + 4) slashing damage.
    """
    name = "Griffon"
    description = "Large monstrosity, unaligned"
    challenge_rating = 2
    armor_class = 12
    skills = "Perception +5"
    senses = "Darkvision 60 ft., Passive Perception 15"
    languages = ""
    strength = Ability(18)
    dexterity = Ability(15)
    constitution = Ability(16)
    intelligence = Ability(2)
    wisdom = Ability(13)
    charisma = Ability(8)
    speed = 30
    swim_speed = 0
    fly_speed = 80
    climb_speed = 0
    hp_max = 59
    hit_dice = "7d10"
    spells = []


class Grimlock(Monster):
    """
    Blind Senses.
      The grimlock can't use its blindsight while deafened and unable to
      smell.
    Keen Hearing and Smell.
      The grimlock has advantage on Wisdom (Perception) checks that rely on
      hearing or smell.
    Stone Camouflage.
      The grimlock has advantage on Dexterity (Stealth) checks made to hide
      in rocky terrain.
    Spiked Bone Club.
      Melee Weapon Attack: +5 to hit, reach 5 ft., one target. Hit: 5 (1d4 +
      3) bludgeoning damage plus 2 (1d4) piercing damage.
    """
    name = "Grimlock"
    description = "Medium humanoid, neutral evil"
    challenge_rating = 0.25
    armor_class = 11
    skills = "Athletics +5, Perception +3, Stealth +3"
    senses = "Blindsight 30 ft. or 10 ft. while deafened (blind beyond this radius), Passive Perception 13"
    languages = "Undercommon"
    strength = Ability(16)
    dexterity = Ability(12)
    constitution = Ability(12)
    intelligence = Ability(9)
    wisdom = Ability(8)
    charisma = Ability(6)
    speed = 30
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 11
    hit_dice = "2d8"
    spells = []


class Guard(Monster):
    """
    Spear.
      Melee or Ranged Weapon Attack: +3 to hit, reach 5 ft. or range 20/60
      ft., one target. Hit: 4 (1d6 + 1) piercing damage or 5 (1d8 + 1)
      piercing damage if used with two hands to make a melee attack.
    """
    name = "Guard"
    description = "Medium humanoid, any alignment"
    challenge_rating = 0.125
    armor_class = 16
    skills = "Perception +2"
    senses = "Passive Perception 12"
    languages = "any one language (usually Common)"
    strength = Ability(13)
    dexterity = Ability(12)
    constitution = Ability(12)
    intelligence = Ability(10)
    wisdom = Ability(11)
    charisma = Ability(10)
    speed = 30
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 11
    hit_dice = "2d8"
    spells = []


class GuardianNaga(Monster):
    """
    Rejuvenation.
      If it dies, the naga returns to life in 1d6 days and regains all its
      hit points. Only a wish spell can prevent this trait from functioning.
    Spellcasting.
      The naga is an 11th-level spellcaster. Its spellcasting ability is
      Wisdom (spell save DC 16, +8 to hit with spell attacks), and it needs
      only verbal components to cast its spells. It has the following cleric
      spells prepared:
      
  
      
      - Cantrips (at will): mending, sacred flame, thaumaturgy
      
      - 1st level (4 slots): command, cure wounds, shield of faith
      
      - 2nd level (3 slots): calm emotions, hold person
      
      - 3rd level (3 slots): bestow curse, clairvoyance
      
      - 4th level (3 slots): banishment, freedom of movement
      
      - 5th level (2 slots): flame strike, geas
      
      - 6th level (1 slot): true seeing
    Bite.
      Melee Weapon Attack: +8 to hit, reach 10 ft., one creature. Hit: 8
      (1d8 + 4) piercing damage, and the target must make a DC 15
      Constitution saving throw, taking 45 (10d8) poison damage on a failed
      save, or half as much damage on a successful one.
    Spit Poison.
      Ranged Weapon Attack: +8 to hit, range 15/30 ft., one creature. Hit:
      The target must make a DC 15 Constitution saving throw, taking 45
      (10d8) poison damage on a failed save, or half as much damage on a
      successful one.
    """
    name = "Guardian Naga"
    description = "Large monstrosity, lawful good"
    challenge_rating = 10
    armor_class = 18
    skills = ""
    senses = "Darkvision 60 ft., Passive Perception 14"
    languages = "Celestial, Common"
    strength = Ability(19)
    dexterity = Ability(18)
    constitution = Ability(16)
    intelligence = Ability(16)
    wisdom = Ability(19)
    charisma = Ability(18)
    speed = 40
    swim_speed = 0
    fly_speed = 0
    climb_speed = 0
    hp_max = 127
    hit_dice = "15d10"
    spells = ["mending", "sacred flame", "thaumaturgy", "command", "cure wounds", "shield of faith", "calm emotions", "hold person", "bestow curse", "clairvoyance", "banishment", "freedom of movement", "flame strike", "geas", "true seeing"]


class Gynosphinx(Monster):
    """
    Inscrutable.
      The sphinx is immune to any effect that would sense its emotions or
      read its thoughts, as well as any divination spell that it refuses.
      Wisdom (Insight) checks made to ascertain the sphinx's intentions or
      sincerity have disadvantage.
    Magic Weapons.
      The sphinx's weapon attacks are magical.
    Spellcasting.
      The sphinx is a 9th-level spellcaster. Its spellcasting ability is
      Intelligence (spell save DC 16, +8 to hit with spell attacks). It
      requires no material components to cast its spells. The sphinx has the
      following wizard spells prepared:
      
  
      
      - Cantrips (at will): mage hand, minor illusion, prestidigitation
      
      - 1st level (4 slots): detect magic, identify, shield
      
      - 2nd level (3 slots): darkness, locate object, suggestion
      
      - 3rd level (3 slots): dispel magic, remove curse, tongues
      
      - 4th level (3 slots): banishment, greater invisibility
      
      - 5th level (1 slot): legend lore
    Multiattack.
      The sphinx makes two claw attacks.
    Claw.
      Melee Weapon Attack: +9 to hit, reach 5 ft., one target. Hit: 13 (2d8
      + 4) slashing damage.
    """
    name = "Gynosphinx"
    description = "Large monstrosity, lawful neutral"
    challenge_rating = 11
    armor_class = 17
    skills = "Arcana +12, History +12, Perception +8, Religion +8"
    senses = "Truesight 120 ft., Passive Perception 18"
    languages = "Common, Sphinx"
    strength = Ability(18)
    dexterity = Ability(15)
    constitution = Ability(16)
    intelligence = Ability(18)
    wisdom = Ability(18)
    charisma = Ability(18)
    speed = 40
    swim_speed = 0
    fly_speed = 60
    climb_speed = 0
    hp_max = 136
    hit_dice = "16d10"
    spells = ["mage hand", "minor illusion", "prestidigitation",
              "detect magic", "identify", "shield", "darkness", "locate object",
              "suggestion", "dispel magic", "remove curse", "tongues",
              "banishment", "greater invisibility", "legend lore"]
